Package: kdensity
Type: Package
Title: Kernel Density Estimation with Parametric Starts and Asymmetric
        Kernels
Version: 1.2.0
Authors@R: c(
    person("Jonas", "Moss", , "jonas.gjertsen@gmail.com", role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-6876-6964")),
    person("Martin", "Tveten", ,"tveten@nr.no", role = c("ctb"))
    )
Description: Handles univariate non-parametric density estimation with 
    parametric starts and asymmetric kernels in a simple and flexible way. 
    Kernel density estimation with parametric starts involves fitting a
    parametric density to the data before making a correction with kernel 
    density estimation, see Hjort & Glad (1995) <doi:10.1214/aos/1176324627>.
    Asymmetric kernels make kernel density estimation more efficient on bounded
    intervals such as (0, 1) and the positive half-line. Supported asymmetric 
    kernels are the gamma kernel of Chen (2000) <doi:10.1023/A:1004165218295>,
    the beta kernel of Chen (1999) <doi:10.1016/S0167-9473(99)00010-9>, and the
    copula kernel of Jones & Henderson (2007) <doi:10.1093/biomet/asm068>.
    User-supplied kernels, parametric starts, and bandwidths are supported.
License: MIT + file LICENSE
URL: https://github.com/JonasMoss/kdensity
BugReports: https://github.com/JonasMoss/kdensity/issues
Encoding: UTF-8
Suggests: extraDistr, testthat, knitr, rmarkdown
Imports: univariateML
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 15:20:52 UTC; jmoss
Author: Jonas Moss [aut, cre] (ORCID: <https://orcid.org/0000-0002-6876-6964>),
  Martin Tveten [ctb]
Maintainer: Jonas Moss <jonas.gjertsen@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 21:10:57 UTC
Built: R 4.5.2; ; 2026-05-04 21:54:06 UTC; unix

Package: kdensity
Type: Package
Title: Kernel Density Estimation with Parametric Starts and Asymmetric
        Kernels
Version: 1.2.0
Authors@R: c(
    person("Jonas", "Moss", , "jonas.gjertsen@gmail.com", role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-6876-6964")),
    person("Martin", "Tveten", ,"tveten@nr.no", role = c("ctb"))
    )
Description: Handles univariate non-parametric density estimation with 
    parametric starts and asymmetric kernels in a simple and flexible way. 
    Kernel density estimation with parametric starts involves fitting a
    parametric density to the data before making a correction with kernel 
    density estimation, see Hjort & Glad (1995) <doi:10.1214/aos/1176324627>.
    Asymmetric kernels make kernel density estimation more efficient on bounded
    intervals such as (0, 1) and the positive half-line. Supported asymmetric 
    kernels are the gamma kernel of Chen (2000) <doi:10.1023/A:1004165218295>,
    the beta kernel of Chen (1999) <doi:10.1016/S0167-9473(99)00010-9>, and the
    copula kernel of Jones & Henderson (2007) <doi:10.1093/biomet/asm068>.
    User-supplied kernels, parametric starts, and bandwidths are supported.
License: MIT + file LICENSE
URL: https://github.com/JonasMoss/kdensity
BugReports: https://github.com/JonasMoss/kdensity/issues
Encoding: UTF-8
Suggests: extraDistr, testthat, knitr, rmarkdown
Imports: univariateML
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 15:20:52 UTC; jmoss
Author: Jonas Moss [aut, cre] (ORCID: <https://orcid.org/0000-0002-6876-6964>),
  Martin Tveten [ctb]
Maintainer: Jonas Moss <jonas.gjertsen@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 21:10:57 UTC
Built: R 4.6.0; ; 2026-05-05 00:20:57 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("$",kdensity)
S3method("$<-",kdensity)
S3method("[[",kdensity)
S3method("[[<-",kdensity)
S3method(coef,kdensity)
S3method(lines,kdensity)
S3method(logLik,kdensity)
S3method(plot,kdensity)
S3method(points,kdensity)
S3method(print,kdensity)
S3method(summary,kdensity)
S3method(update,kdensity)
export(kdensity)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# kdensity <img src="man/figures/logo.png" align="right" width="247" height="70" />

[![R build
status](https://github.com/JonasMoss/kdensity/workflows/R-CMD-check/badge.svg)](https://github.com/JonasMoss/kdensity/actions)
[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/kdensity)](https://cran.r-project.org/package=kdensity)
[![DOI](https://zenodo.org/badge/120678148.svg)](https://zenodo.org/badge/latestdoi/120678148)

An `R` package for univariate kernel density estimation with parametric
starts and asymmetric kernels.

## News

`kdensity` is now linked to `univariateML`, meaning it supports the
approximately 30+ parametric starts from that package\!

## Overview

kdensity is an implementation of univariate kernel density estimation
with support for parametric starts and asymmetric kernels. Its main
function is `kdensity`, which is has approximately the same syntax as
`stats::density`. Its new functionality is:

  - `kdensity` has built-in support for many *parametric starts*, such
    as `normal` and `gamma`, but you can also supply your own. For a
    list of supported parametric starts, see the readme of
    [`univariateML`](https://github.com/JonasMoss/univariateML).
  - It supports several asymmetric kernels ones such as `gcopula` and
    `gamma` kernels, but also the common symmetric ones. In addition,
    you can also supply your own kernels.
  - A selection of choices for the bandwidth function `bw`, again
    including an option to specify your own.
  - The returned value is density function. This can be used for
    e.g. numerical integration, numerical differentiation, and point
    evaluations.

A reason to use `kdensity` is to avoid *boundary bias* when estimating
densities on the unit interval or the positive half-line. Asymmetric
kernels such as `gamma` and `gcopula` are designed for this purpose. The
support for parametric starts allows you to easily use a method that is
often superior to ordinary kernel density estimation.

Several `R` packages deal with kernel estimation. For an overview see
[Deng & Hadley Wickham
(2011)](https://vita.had.co.nz/papers/density-estimation.pdf). While no
other `R` package handles density estimation with parametric starts,
several packages supports methods that handle boundary bias.
[`evmix`](http://www.math.canterbury.ac.nz/~c.scarrott/evmix/) provides
a variety of boundary bias correction methods in the `bckden` function.
[`kde1d`](https://github.com/tnagler/kde1d) corrects for boundary bias
using transformed univariate local polynomial kernel density estimation.
[`logKDE`](https://github.com/andrewthomasjones/logKDE) corrects for
boundary bias on the half line using a logarithmic transform.
[`ks`](https://CRAN.R-project.org/package=ks) supports boundary
correction through the `kde.boundary` function, while
[`Ake`](https://CRAN.R-project.org/package=Ake) corrects for boundary
bias using tailored kernel functions.

## Installation

From inside `R`, use one of the following commands:

``` r
# For the CRAN release
install.packages("kdensity")
# For the development version from GitHub:
# install.packages("devtools")
devtools::install_github("JonasMoss/kdensity")
```

## Usage Example

Call the `library` function and use it just like `stats::density`, but
with optional additional arguments.

``` r
library("kdensity")
plot(kdensity(mtcars$mpg, start = "normal"))
```

## Description

Kernel density estimation with a *parametric start* was introduced by
Hjort and Glad in [Nonparametric Density Estimation with a Parametric
Start (1995)](https://projecteuclid.org/euclid.aos/1176324627). The idea
is to start out with a parametric density before you do your kernel
density estimation, so that your actual kernel density estimation will
be a correction to the original parametric estimate. The resulting
estimator will outperform the ordinary kernel density estimator in terms
of asymptotic integrated mean squared error whenever the true density is
close to your suggestion; and the estimator can be superior to the
ordinary kernel density estimator even when the suggestion is pretty far
off.

In addition to parametric starts, the package implements some
*asymmetric kernels*. These kernels are useful when modelling data with
sharp boundaries, such as data supported on the positive half-line or
the unit interval. Currently we support the following asymmetric
kernels:

  - Jones and Henderson’s *Gaussian copula KDE*, from Kernel-Type
    Density Estimation on the Unit Interval
    (2007)).
    This is used for data on the unit interval. The bandwidth selection
    mechanism described in that paper is implemented as well. This
    kernel is called `gcopula`.

  - Chen’s two *beta kernels* from Beta kernel estimators for density
    functions
    (1999).
    These are used for data supported on the on the unit interval, and
    are called `beta` and `beta_biased`.

  - Chen’s two *gamma kernels* from Probability Density Function
    Estimation Using Gamma Kernels
    (2000).
    These are used for data supported on the positive half-line, and are
    called `gamma` and `gamma_biased`.

These features can be combined to make asymmetric kernel densities
estimators with parametric starts, see the example below. The package
contains only one function, `kdensity`, in addition to the generics
`plot`, `points`, `lines`, `summary`, and `print`.

## Usage

The function `kdensity` takes some `data`, a kernel `kernel` and a
parametric start `start`. You can optionally specify the `support`
parameter, which is used to find the normalizing constant.

The following example uses the  data set. The black curve is a
gamma-kernel density estimate with a gamma start, the red curve a fully
parametric gamma density and and the blue curve an ordinary `density`
estimate. Notice the boundary bias of the ordinary `density` estimator.
The underlying parameter estimates are always maximum likelilood.

``` r
library("kdensity")
kde = kdensity(airquality$Wind, start = "gamma", kernel = "gamma")
plot(kde, main = "Wind speed (mph)")
lines(kde, plot_start = TRUE, col = "red")
lines(density(airquality$Wind, adjust = 2), col = "blue")
rug(airquality$Wind)
```

<img src="man/figures/README-example-1.png" width="750px" />

Since the return value of `kdensity` is a function, `kde` is callable
and can be used as any density function in `R` (such as `stats::dnorm`).
For example, you can do:

``` r
kde(10)
#> [1] 0.09980471
integrate(kde, lower = 0, upper = 1) # The cumulative distribution up to 1.
#> 1.27532e-05 with absolute error < 2.2e-19
```

You can access the parameter estimates by using `coef`. You can also
access the log likelihood (`logLik`), AIC and BIC of the parametric
start distribution.

``` r
coef(kde)
#> Maximum likelihood estimates for the Gamma model 
#>  shape    rate  
#> 7.1873  0.7218
logLik(kde)
#> 'log Lik.' 12.33787 (df=2)
AIC(kde)
#> [1] -20.67574
```

## How to Contribute or Get Help

If you encounter a bug, have a feature request or need some help, open a
[Github issue](https://github.com/JonasMoss/kdensity/issues). Create a
pull requests to contribute. This project follows a [Contributor Code of
Conduct](https://www.contributor-covenant.org/version/1/4/code-of-conduct/).

## References

  - [Hjort, Nils Lid, and Ingrid K. Glad. “Nonparametric density
    estimation with a parametric start.” The Annals of Statistics
    (1995): 882-904.](https://projecteuclid.org/euclid.aos/1176324627).

  - [Jones, M. C., and D. A. Henderson. “Miscellanea kernel-type density
    estimation on the unit interval.” Biometrika 94.4 (2007):977-984.]

  - [Chen, Song Xi. “Probability density function estimation using gamma
    kernels.” Annals of the Institute of Statistical Mathematics 52.3
    (2000):
    471-480.](https://link.springer.com/article/10.1023/A:1004165218295).

  - [Chen, Song Xi. “Beta kernel estimators for density functions.”
    Computational Statistics & Data Analysis 31.2 (1999):131-145.]

<!-- README.md is generated from README.Rmd. Please edit that file -->

# kdensity <img src="man/figures/logo.png" align="right" width="247" height="70" />

[![R build
status](https://github.com/JonasMoss/kdensity/workflows/R-CMD-check/badge.svg)](https://github.com/JonasMoss/kdensity/actions)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/kdensity)](https://cran.r-project.org/package=kdensity)
[![DOI](https://zenodo.org/badge/120678148.svg)](https://zenodo.org/badge/latestdoi/120678148)

An `R` package for univariate kernel density estimation with parametric
starts and asymmetric kernels.

## News

For the bias-corrected `beta` kernel with a uniform or constant start,
`kdensity` now defaults to a new closed-form bandwidth selector, `"HS"`,
from [Hallberg Szabadváry’s *A Fast, Closed-Form Bandwidth Selector for
the Beta Kernel Density Estimator*
(2026)](https://arxiv.org/abs/2601.19553). It is much faster than the
previous `"ucv"` default.

## Overview

kdensity is an implementation of univariate kernel density estimation
with support for parametric starts and asymmetric kernels. Its main
function is `kdensity`, which is has approximately the same syntax as
`stats::density`. Its new functionality is:

- `kdensity` has built-in support for many *parametric starts*, such as
  `normal` and `gamma`, but you can also supply your own. For a list of
  supported parametric starts, see the readme of
  [`univariateML`](https://github.com/JonasMoss/univariateML).
- It supports several asymmetric kernels ones such as `gcopula` and
  `gamma` kernels, but also the common symmetric ones. In addition, you
  can also supply your own kernels.
- A selection of choices for the bandwidth function `bw`, again
  including an option to specify your own.
- The returned value is density function. This can be used for
  e.g. numerical integration, numerical differentiation, and point
  evaluations.

A reason to use `kdensity` is to avoid *boundary bias* when estimating
densities on the unit interval or the positive half-line. Asymmetric
kernels such as `gamma` and `gcopula` are designed for this purpose. The
support for parametric starts allows you to easily use a method that is
often superior to ordinary kernel density estimation.

Several `R` packages deal with kernel estimation. For an overview see
[Deng & Hadley Wickham
(2011)](https://vita.had.co.nz/papers/density-estimation.pdf). While no
other `R` package handles density estimation with parametric starts,
several packages supports methods that handle boundary bias.
[`evmix`](http://www.math.canterbury.ac.nz/~c.scarrott/evmix/) provides
a variety of boundary bias correction methods in the `bckden` function.
[`kde1d`](https://github.com/tnagler/kde1d) corrects for boundary bias
using transformed univariate local polynomial kernel density estimation.
[`logKDE`](https://github.com/andrewthomasjones/logKDE) corrects for
boundary bias on the half line using a logarithmic transform.
[`ks`](https://CRAN.R-project.org/package=ks) supports boundary
correction through the `kde.boundary` function, while
[`Ake`](https://CRAN.R-project.org/package=Ake) corrects for boundary
bias using tailored kernel functions.

## Installation

From inside `R`, use one of the following commands:

``` r
# For the CRAN release
install.packages("kdensity")
# For the development version from GitHub:
# install.packages("devtools")
devtools::install_github("JonasMoss/kdensity")
```

## Usage Example

Call the `library` function and use it just like `stats::density`, but
with optional additional arguments.

``` r
library("kdensity")
plot(kdensity(mtcars$mpg, start = "normal"))
```

## Description

Kernel density estimation with a *parametric start* was introduced by
Hjort and Glad in [Nonparametric Density Estimation with a Parametric
Start (1995)](https://projecteuclid.org/euclid.aos/1176324627). The idea
is to start out with a parametric density before you do your kernel
density estimation, so that your actual kernel density estimation will
be a correction to the original parametric estimate. The resulting
estimator will outperform the ordinary kernel density estimator in terms
of asymptotic integrated mean squared error whenever the true density is
close to your suggestion; and the estimator can be superior to the
ordinary kernel density estimator even when the suggestion is pretty far
off.

In addition to parametric starts, the package implements some
*asymmetric kernels*. These kernels are useful when modelling data with
sharp boundaries, such as data supported on the positive half-line or
the unit interval. Currently we support the following asymmetric
kernels:

- Jones and Henderson’s *Gaussian copula KDE*, from [Kernel-Type Density
  Estimation on the Unit Interval
  (2007)](https://academic.oup.com/biomet/article-abstract/94/4/977/246269).
  This is used for data on the unit interval. The bandwidth selection
  mechanism described in that paper is implemented as well. This kernel
  is called `gcopula`.

- Chen’s two *beta kernels* from [Beta kernel estimators for density
  functions
  (1999)](https://www.sciencedirect.com/science/article/pii/S0167947399000109).
  These are used for data supported on the on the unit interval, and are
  called `beta` and `beta_biased`.

- Chen’s two *gamma kernels* from [Probability Density Function
  Estimation Using Gamma Kernels
  (2000)](https://link.springer.com/article/10.1023/A:1004165218295).
  These are used for data supported on the positive half-line, and are
  called `gamma` and `gamma_biased`.

These features can be combined to make asymmetric kernel densities
estimators with parametric starts, see the example below. The package
contains only one function, `kdensity`, in addition to the generics
`plot`, `points`, `lines`, `summary`, and `print`.

``` r
set.seed(42)
beta_fit <- kdensity(rbeta(250, 2, 5), kernel = "beta")
beta_fit$bw_str
#> [1] "HS"
beta_fit$bw
#> [1] 0.02052664
```

## Usage

The function `kdensity` takes some `data`, a kernel `kernel` and a
parametric start `start`. You can optionally specify the `support`
parameter, which is used to find the normalizing constant.

The following example uses the data set. The black curve is a
gamma-kernel density estimate with a gamma start, the red curve a fully
parametric gamma density and and the blue curve an ordinary `density`
estimate. Notice the boundary bias of the ordinary `density` estimator.
The underlying parameter estimates are always maximum likelilood.

``` r
library("kdensity")
kde <- kdensity(airquality$Wind, start = "gamma", kernel = "gamma")
plot(kde, main = "Wind speed (mph)")
lines(kde, plot_start = TRUE, col = "red")
lines(density(airquality$Wind, adjust = 2), col = "blue")
rug(airquality$Wind)
```

<img src="man/figures/README-example-1.png" width="750px" />

Since the return value of `kdensity` is a function, `kde` is callable
and can be used as any density function in `R` (such as `stats::dnorm`).
For example, you can do:

``` r
kde(10)
#> [1] 0.09980471
integrate(kde, lower = 0, upper = 1) # The cumulative distribution up to 1.
#> 1.27532e-05 with absolute error < 2.2e-19
```

You can access the parameter estimates by using `coef`. You can also
access the log likelihood (`logLik`), AIC and BIC of the parametric
start distribution.

``` r
coef(kde)
#> Maximum likelihood estimates for the Gamma model 
#>  shape    rate  
#> 7.1873  0.7218
logLik(kde)
#> 'log Lik.' 12.33787 (df=2)
AIC(kde)
#> [1] -20.67574
```

## How to Contribute or Get Help

If you encounter a bug, have a feature request or need some help, open a
[Github issue](https://github.com/JonasMoss/kdensity/issues). Create a
pull requests to contribute. This project follows a [Contributor Code of
Conduct](https://www.contributor-covenant.org/version/1/4/code-of-conduct/).

## References

- [Hjort, Nils Lid, and Ingrid K. Glad. “Nonparametric density
  estimation with a parametric start.” The Annals of Statistics (1995):
  882-904.](https://projecteuclid.org/euclid.aos/1176324627).

- \[Jones, M. C., and D. A. Henderson. “Miscellanea kernel-type density
  estimation on the unit interval.” Biometrika 94.4 (2007): 977-984.\].

- [Chen, Song Xi. “Probability density function estimation using gamma
  kernels.” Annals of the Institute of Statistical Mathematics 52.3
  (2000):
  471-480.](https://link.springer.com/article/10.1023/A:1004165218295).

- \[Chen, Song Xi. “Beta kernel estimators for density functions.”
  Computational Statistics & Data Analysis 31.2 (1999): 131-145.\]

- [Hallberg Szabadváry, Johan. “A Fast, Closed-Form Bandwidth Selector
  for the Beta Kernel Density Estimator.” arXiv preprint
  arXiv:2601.19553 (2026).](https://arxiv.org/abs/2601.19553)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----mpg_example_start, echo = TRUE-------------------------------------------
library("kdensity")
kde <- kdensity(mtcars$mpg, start = "gumbel", kernel = "gaussian")
kde(20)

## ----mpg_example_expectation, echo = TRUE-------------------------------------
integrate(function(x) x * kde(x), lower = -Inf, upper = Inf)$value

## ----mpg_example_plot, echo = TRUE,fig.height = 5, fig.width = 5, fig.align = "center"----
plot(kde, main = "Miles per Gallon")
lines(kde, plot_start = TRUE, col = "red")
rug(mtcars$mpg)

## ----mpg_example_plot_magrittr, echo = TRUE, eval = FALSE---------------------
# library("magrittr")
# kde %>%
#   plot(main = "Miles per Gallon") %>%
#   lines(plot_start = TRUE, col = "red")

## ----mpg_example_generics, echo = TRUE----------------------------------------
coef(kde)
logLik(kde)
AIC(kde)

## ----mpg_example_summary, echo = TRUE-----------------------------------------
summary(kde)

## ----mpg_example_access, echo = TRUE------------------------------------------
kde$start_str
kde[["x"]]

## ----mpg_example_changes, echo = TRUE-----------------------------------------
kde$bw <- "RHE"
update(kde, start = "normal")

## -----------------------------------------------------------------------------
normal <- list(
  density = dnorm,
  estimator = function(data) {
    c(
      mean = mean(data),
      sd = sd(data)
    )
  },
  support = c(-Inf, Inf)
)

## ----fig2, fig.height = 5, fig.width = 5, fig.align = "center"----------------
LH <- diff(LakeHuron)
kde_normal <- kdensity(LH, start = "normal")
plot(kde_normal,
  lwd = 2, col = "black",
  main = "Lake Huron differences"
)

## -----------------------------------------------------------------------------
gaussian <- list(
  kernel  = function(y, x, h) dnorm((y - x) / h),
  sd      = 1,
  support = c(-Inf, Inf)
)

#' Parametrically guided kernel density estimation
#'
#' `kdensity` computes a parametrically guided kernel density estimate
#' for univariate data. It supports asymmetric kernels and parametric starts
#' through the `kernel` and `start` arguments.
#'
#' The default values for `bw`, `kernel`, `start`, and
#' `support` are interdependent, and are chosen to make sense. E.g.,
#' the default value for `support` when `start = beta` is
#' `c(0, 1)`.
#'
#' The `start` argument defaults to `uniform`, which corresponds
#' to ordinary kernel density estimation. The typical default value for
#' `kernel` is `gaussian`.
#'
#' @export
#'
#' @param x Numeric vector containing the data.
#'
#' @param bw A bandwidth function. Can be either a string, a custom-made
#' function, or a double. The supported bandwidth functions are documented
#' in [bandwidths()].
#'
#' @param adjust An adjustment constant, so that `h = adjust*bw*sd`, where `sd`
#' varies with the chosen kernel.
#'
#' @param kernel The kernel function. Can be chosen from the list of built-in
#' kernels or be custom-made. See [kernels()] for details.
#'
#' @param start Parametric start. Can be chosen from the list of built-in
#' parametric starts or be custom-made. See [parametric_starts()] for
#' details.
#'
#' @param support The support of the data. Must be compatible with the supplied
#' `x` and the supplied `start` and `kernel`. Is used to find the
#' normalization constant, see `normalized`.
#'
#' @param na.rm Logical; if `TRUE`, `NA`s will be removed from `x`.
#'
#' @param normalized Logical; if `TRUE`, the density is normalized.
#'
#' @param tolerance Numeric; the relative error to tolerate in normalization.
#'
#' @return `kdensity` returns an S3 function object of
#' [base::class()] "kdensity". This is a callable function with the
#' following elements, accessible by '$':
#' \describe{
#'   \item{`x`}{The data supplied in `x`.}
#'   \item{`bw_str, bw, adjust, h`}{The bandwidth function, the resulting
#'               bandwidth, the `adjust` argument, and the adjusted
#'               bandwidth.}
#'   \item{`kernel_str, kernel, start, start_str, support`}{Name of the kernel,
#'               the kernel object, name of the parametric start, the start object,
#'               and the support of the density.}
#'   \item{`data.name, n, range, has.na, na.rm, normalized`}{Name of the data, number of
#'               observations, the range of the data, whether the data
#'               `x` contained `NA` values, whether na.rm is `TRUE`
#'               or not, and whether the density is normalized.}
#'   \item{`call`}{The `call` to `kdensity`.}
#'   \item{`estimates`}{Named numeric vector containing the parameter
#'               estimates from the parametric start.}
#'   \item{`logLik`}{The log-likelihood of the parametric starts. Is `NA`
#'               for the uniform start.}
#'
#' }
#'
#'
#' @details If `normalized` is `FALSE` and `start != "uniform"`, the resulting
#' density will not integrate to 1 in general.
#'
#' @seealso The `stats` package function [stats::density()].
#' @references
#'   Hjort, Nils Lid, and Ingrid K. Glad. "Nonparametric density estimation with a parametric start." The Annals of Statistics (1995): 882-904.
#'
#'   Jones, M. C., and D. A. Henderson. "Miscellanea kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.
#'
#'   Chen, Song Xi. "Probability density function estimation using gamma kernels." Annals of the Institute of Statistical Mathematics 52.3 (2000): 471-480.
#'
#'   Silverman, Bernard W. Density estimation for statistics and data analysis. Vol. 26. CRC press, 1986.
#'
#' @examples
#' ## Use gamma kernels to model positive data, the concentration of
#' ## theophylline
#'
#' concentration <- Theoph$conc + 0.001
#' plot(kdensity(concentration, start = "gamma", kernel = "gamma", adjust = 1 / 3),
#'   ylim = c(0, 0.15), lwd = 2, main = "Concentration of theophylline"
#' )
#' lines(kdensity(concentration, start = "gamma", kernel = "gaussian"),
#'   lty = 2, col = "grey", lwd = 2
#' )
#' lines(kdensity(concentration, start = "gaussian", kernel = "gaussian"),
#'   lty = 3, col = "blue", lwd = 2
#' )
#' lines(kdensity(concentration, start = "gaussian", kernel = "gamma", adjust = 1 / 3),
#'   lty = 4, col = "red", lwd = 2
#' )
#' rug(concentration)
#'
kdensity <- function(x, bw = NULL, adjust = 1, kernel = NULL, start = NULL,
                     support = NULL, na.rm = FALSE, normalized = TRUE,
                     tolerance = 0.01) {
  ## These tests are based purely on the data, and should be run no matter what.
  data.name <- deparse(substitute(x))
  has.na <- anyNA(x)

  assert_(!(has.na & !na.rm),
    msg = "x contains NAs and na.rm = FALSE."
  )

  x <- x[!is.na(x)] # This line is reached only if (has.na & !na.rm) is FALSE.

  ## 'kernel', 'start' and 'bw' can be custom made: In this case, they must
  ## be added to their environments.

  if (!is.null(start)) {
    if (!is.character(start)) {
      start_str <- deparse(substitute(start))
      add_start(start_str = start_str, start = start)
      start <- start_str
    }
  }

  if (!is.null(kernel)) {
    if (!is.character(kernel)) {
      kernel_str <- deparse(substitute(kernel))
      add_kernel(kernel_str = kernel_str, kernel = kernel)
      kernel <- kernel_str
    }
  }

  ## The case of bw == Inf is special! In this case, we return the parametric
  ## start itself.

  ## Now we handle the combinations of kernel, start and support.
  ## This is fancy defaults management.
  kss_list <- get_kernel_start_support(kernel, start, support)

  ## The strings are used for reporting and inside bandwidth functions,
  ## at must be preserved.
  start_str <- kss_list$start_str
  kernel_str <- kss_list$kernel_str

  ## We overwrite the kernel, start, and support with what we obtained from kss.
  kernel <- kss_list$kernel
  start <- kss_list$start
  support <- kss_list$support

  ## Tests for incompabibilities in the supplied values.
  support_compatible(kernel, start, support)

  ## We continue by checking if the supplied values make sense.
  if (!all(x <= support[2] & x >= support[1])) {
    stop(
      "The supplied data x is not contained in the support: (",
      support[1], ", ", support[2], ")."
    )
  }

  estimates <- start$estimator(x)
  parametric_start <- start$density

  # Name of the variable where the density is evaluated. Typically x.
  x_name <- names(formals(start$density))[1]

  parametric_start_vector <- function(x) {
    arguments <- list()
    arguments[[1]] <- x
    names(arguments)[1] <- x_name
    arguments <- append(arguments, as.list(estimates))
    do.call(parametric_start, arguments)
  }

  ## Takes care of the bandwidth. Can be either a double, a string, or a
  ## function taking the arguments data, kernel, start support.

  if (is.null(bw)) {
    bw <- get_standard_bw(kernel_str, start_str, support)
  }

  if (!is.numeric(bw)) {
    if (is.character(bw)) {
      bw_str <- bw
      bw <- get_bw(bw)(x, kernel_str, start_str, support)
    } else {
      bw_str <- deparse(substitute(bw))
      bw <- bw(x, kernel_str, start_str, support)
    }
  } else {
    bw_str <- "user supplied"
    if (bw == Inf) {
      msg <- "bw = Inf does not work with a uniform start."
      assert_(start_str != "uniform", start_str != "constant", msg = msg)
    }
  }

  ## The parameter h is computed. The basic bandwidth is h = bw*adjust for the
  ## normal kernel, and is adjusted for all the other kernels so that the sd
  ## of the kernel equals h.
  sd <- ifelse(!is.null(kss_list$kernel$sd), kss_list$kernel$sd, 1)
  h <- bw * adjust * sd

  ## The denominator can be computed.
  parametric_start_data <- parametric_start_vector(x)

  ## Now we handle normalization.

  normalization <- 1

  if (normalized & bw != Inf) {
    integrand <- function(y) {
      sapply(y, function(y) {
        mean(1 / h * kernel$kernel(y, x, h) /
          parametric_start_data) *
          parametric_start_vector(y)
      })
    }

    integral <- tryCatch(
      stats::integrate(integrand,
        lower = support[1],
        upper = support[2]
      ),
      error = function(e) {
        stop(paste0(
          "Normalization error: The function will not integrate.",
          "Two common causes are: 1.) The kernel is non-smooth, ",
          "try a smooth kernel if possible. 2.) The supplied ",
          "support is incorrect."
        ))
      }
    )

    msg <- "The normalization constant has too large relative error.
   1.) try to normalize the data, or
   2.) change kernel, or
   3.) increase the 'tolerance' parameter for kdensity"

    assert_(!is.nan(integral$abs.error / integral$value),
      msg = msg
    )
    assert_(integral$abs.error / integral$value < tolerance,
      msg = msg
    )

    normalization <- integral$value
  }


  ## This is the main part of the returned object.

  return_function <- function(y) {
    if (missing(y)) {
      stop("A kdensity object is a density function. Call it with numerical input.")
    }

    if (h != Inf) {
      if (length(y) == 1) {
        mean(1 / h * kernel$kernel(y, x, h) * parametric_start_vector(y) /
          parametric_start_data) / normalization
      } else {
        sapply(y, function(y) {
          parametric_start_vector_y <- parametric_start_vector(y)
          1 / h * mean(kernel$kernel(y, x, h) * parametric_start_vector_y /
            parametric_start_data) / normalization
        })
      }
    } else {
      ## If bw = Inf, the parametric start is returned.
      arguments <- list()
      arguments[[1]] <- y
      names(arguments)[1] <- x_name
      arguments <- append(arguments, as.list(estimates))
      do.call(parametric_start, arguments)
    }
  }

  ## 'R6'-ish attributes:
  call <- match.call()
  range <- c(min(x), max(x))
  n <- length(x)
  logLik <- ifelse(start_str == "uniform" | start_str == "constant", NA,
    sum(parametric_start_vector(x))
  )

  ## S3 attributes:
  class(return_function) <- "kdensity"
  attr(return_function, "srcref") <- NULL
  return_function
}

context("kdensity helper (default arguments)")
expect_equal(kdensity(precip, support = c(-Inf, Inf))$kernel_str, "gaussian")
expect_equal(kdensity(precip, support = c(0, Inf))$kernel_str, "gamma")
expect_message(kdensity(precip, support = c(7, 999)))
expect_equal(kdensity(precip, support = c(7, 999))$kernel_str, "gamma")
expect_equal(kdensity(precip, support = c(-5, 999))$kernel_str, "gaussian")
set.seed(313)
expect_equal(kdensity(rbeta(10, 10, 10), support = c(0.1, 0.9))$kernel_str, "gcopula")

set.seed(313)
beta_fit <- kdensity(rbeta(50, 2, 5), kernel = "beta")
expect_equal(beta_fit$bw_str, "HS")
expect_gt(beta_fit$bw, 0)

context("return value of generics")
obj <- kdensity(precip, start = "gaussian")
expect_identical(print(obj), obj)
expect_identical(summary(obj), obj)
expect_identical(plot(obj), obj)
expect_identical(plot(obj, plot_start = TRUE), obj)
expect_identical(points(obj), obj)
expect_identical(lines(obj), obj)

context("generics")

obj <- kdensity(mtcars$mpg, kernel = "gaussian", start = "gumbel")

expect_equal(obj$h, obj[["h"]])
expect_s3_class(logLik(obj), "logLik")

expect_equal(
  update(obj, x = 1:100)(1:10),
  kdensity(1:100, kernel = "gaussian", start = "gumbel")(1:10)
)

context("kdensity")

# Normalization errors.
x <- c(170, 172, 142, 199, 180, 184)
expect_error(kdensity(x, start = "normal"))
expect_error(kdensity(x, kernel = "epanechnikov"))
set.seed(1)
x <- rweibull(100, 2, 7)
expect_error(kdensity(x, kernel = "triangular", start = "weibull"))

expect_error(kdensity(c(precip, NA)), "x contains NAs and na.rm = FALSE.")
expect_equal(kdensity(c(precip, NA), na.rm = TRUE)(10:20), kdensity(precip)(10:20))
normal2 <- list(
  density = dnorm,
  estimator = function(data) {
    c(
      mean = mean(data),
      sd = sqrt(stats::var(data) * (length(data) - 1) / length(data))
    )
  },
  support = c(-Inf, Inf)
)

gaussian2 <- list(
  kernel  = function(y, x, h) dnorm((y - x) / h),
  sd      = 1,
  support = c(-Inf, Inf)
)

set.seed(313)
silly_width <- function(x, kernel = NULL, start = NULL, support = NULL) {
  1
}

expect_equal(
  kdensity(precip, start = "normal")(10),
  kdensity(precip, start = normal2)(10)
)
expect_equal(
  kdensity(precip, start = "normal", kernel = "gaussian")(10),
  kdensity(precip, start = normal2, kernel = gaussian2)(10)
)
expect_error(kdensity(precip, kernel = "beta"))
expect_error(kdensity(precip, support = c(0, 1)))
expect_error(kdensity(precip, bw = Inf))
expect_equal(
  kdensity(precip, bw = Inf, start = "normal")(10),
  dnorm(10,
    mean = mean(precip),
    sd = sqrt(stats::var(precip) * (length(precip) - 1) / length(precip))
  )
)
expect_equal(
  kdensity(precip, bw = 1)(10),
  kdensity(precip, bw = silly_width)(10)
)
expect_error(kdensity(precip)())
expect_error(kdensity(precip,
  start = "gumbel", kernel = "rectangular",
  bw = "RHE"
))

context("assignments and updates")

cars$dist %>%
  kdensity(kernel = "gamma") ->
obj

expect_equal(obj$start_str, "uniform")

update(obj, start = "gamma")
expect_equal(obj$start_str, "gamma")

obj[["start"]] <- "gaussian"
expect_equal(obj$start_str, "gaussian")

obj$start <- "gumbel"
expect_equal(obj$start_str, "gumbel")

## Other settable parameters round-trip through update().
obj[["kernel"]] <- "gaussian"
expect_equal(obj$kernel_str, "gaussian")

obj[["adjust"]] <- 2
expect_equal(obj$adjust, 2)

obj[["bw"]] <- "nrd0"
expect_equal(obj$bw_str, "nrd0")

obj[["normalized"]] <- FALSE
expect_false(obj$normalized)

kernel_environment <- new.env(hash = FALSE)

#' Kernel functions
#'
#' Kernel functions are an important part of `kdensity`. This document
#' lists the available built-in functions and the structure of them. Any kernel
#' in the list can be used in `kdensity` by using `kernel = "kernel"`
#' for the intended kernel.
#'
#' Be careful combining kernels with compact support with parametric starts,
#' as the normalizing integral typically fails to converge. Use `gaussian`
#' instead.
#'
#' @section Symmetric kernels:
#' @section Asymmetric kernels:
#' @usage NULL
#' @format NULL
#' @section Structure:
#'    A kernel is a list containing two mandatory elements and one optional
#'    element. The mandatory element '`kernel`' is the kernel function.
#'    It takes arguments `y, x, h`, where `x` is the data supplied
#'    to `kdensity` and `y` is the point of evaluation. `h` is
#'    the bandwidth. Internally, the kernel function is evaluated as
#'    `1/h*kernel(y, x, h)`. It should be vectorized in `x`, but
#'    vectorization in `y` is not needed.
#'
#'    The second mandatory element is `support`, stating the domain of
#'    definition for the kernel. This is used to distinguish kernels on the
#'    unit interval / positive half-line from kernels on R.
#'
#'    `sd` is used for symmetric kernels, and states the standard error
#'    of the kernel. This is used to make kernels comparable to the Gaussian
#'    kernel when calculating bandwidths.
#' @examples
#' gaussian <- list(
#'   kernel = function(y, x, h) stats::dnorm((y - x) / h),
#'   sd = 1,
#'   support = c(-Inf, Inf)
#' )
#'
#' gcopula <- list(
#'   kernel = function(y, x, h) {
#'     rho <- 1 - h^2
#'     inside <- rho^2 * (qnorm(y)^2 + qnorm(x)^2) - 2 * rho * qnorm(y) * qnorm(x)
#'     exp(-inside / (2 * (1 - rho^2)))
#'   },
#'   support = c(0, 1)
#' )
#'
#' @seealso [kdensity()]; [parametric_starts()];
#' [bandwidths()].
#'
#' @section Symmetric kernels:
#'    `gaussian, normal`: The Gaussian kernel. The default argument when
#'    `starts` is supported on R.
#'    `epanechnikov, rectangular (uniform), triangular, biweight,
#'    cosine, optcosine`: Standard symmetric kernels, also used in
#'    [stats::density()].
#'    `tricube, triweight`: Standard symmetric kernels. Not supported by
#'    [stats::density()].
#'    `laplace`: Uses the Laplace density, also known as the double
#'    exponential density.
#' @section Asymmetric kernels:
#'    `gamma, gamma_biased`: The gamma kernel of Chen (2000). For use on the positive
#'    half-line. `gamma` is the recommended biased-corrected kernel.
#'    `gcopula`: The Gaussian copula kernel of Jones & Henderson (2007). For use
#'    on the unit interval.
#'    `beta, beta_biased`: The beta kernel of Chen (1999). For use on the unit interval.
#'    `beta` is the recommended bias-corrected kernel.
#' @references Chen, Song Xi. "Probability density function estimation using gamma kernels." Annals of the Institute of Statistical Mathematics 52.3 (2000): 471-480.
#' Jones, M. C., and D. A. Henderson. "Kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.
#' Chen, Song Xi. "Beta kernel estimators for density functions." Computational Statistics & Data Analysis 31.2 (1999): 131-145.
#' @name kernels
NULL

kernel_environment$gaussian <- list(
  kernel  = function(y, x, h) stats::dnorm((y - x) / h),
  sd      = 1,
  support = c(-Inf, Inf)
)

kernel_environment$normal <- kernel_environment$gaussian

kernel_environment$epanechnikov <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    3 / 4 * (1 - u^2) * (abs(u) <= 1)
  },
  sd = sqrt(5),
  support = c(-Inf, Inf)
)

kernel_environment$rectangular <- list(
  kernel  = function(y, x, h) dunif((x - y) / h, min = -1, max = 1),
  sd      = sqrt(3),
  support = c(-Inf, Inf)
)

kernel_environment$uniform <- kernel_environment$rectangular

kernel_environment$triangular <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    (1 - abs(u)) * (abs(u) <= 1)
  },
  sd = sqrt(6),
  support = c(-Inf, Inf)
)

kernel_environment$biweight <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    15 / 16 * (1 - u^2)^2 * (abs(u) <= 1)
  },
  sd = sqrt(7),
  support = c(-Inf, Inf)
)

kernel_environment$cosine <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    (1 + cos(pi * u)) / 2 * (abs(u) <= 1)
  },
  sd = 1 / sqrt(1 / 3 - 2 / pi^2),
  support = c(-Inf, Inf)
)

kernel_environment$optcosine <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    pi / 4 * cos(pi / 2 * u) * (abs(u) <= 1)
  },
  sd = 1 / sqrt(1 - 8 / pi^2),
  support = c(-Inf, Inf)
)

kernel_environment$triweight <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    35 / 32 * (1 - u^2)^3 * (abs(u) <= 1)
  },
  sd = 3,
  support = c(-Inf, Inf)
)

kernel_environment$tricube <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    70 / 81 * (1 - abs(u)^3)^3 * (abs(u) <= 1)
  },
  sd = 3^(5 / 2) / sqrt(35),
  support = c(-Inf, Inf)
)


kernel_environment$laplace <- list(
  kernel = function(y, x, h) {
    u <- (x - y) / h
    1 / 2 * exp(-abs(u))
  },
  sd = 1 / sqrt(2),
  support = c(-Inf, Inf)
)

kernel_environment$gcopula <- list(
  kernel = function(y, x, h) {
    rho <- 1 - h^2
    inside <- rho^2 * (qnorm(y)^2 + qnorm(x)^2) - 2 * rho * qnorm(y) * qnorm(x)
    exp(-inside / (2 * (1 - rho^2)))
  },
  support = c(0, 1)
)

kernel_environment$gamma <- list(
  kernel = function(y, x, h) {
    indices <- (y >= 2 * h)
    y_new <- y / h * indices + (1 / 4 * (y / h)^2 + 1) * (!indices)
    h * dgamma(x, y_new + 1, scale = h)
  },
  support = c(0, Inf)
)

kernel_environment$gamma_biased <- list(
  kernel = function(y, x, h) h * dgamma(x, y / h + 1, scale = h),
  support = c(0, Inf)
)

kernel_environment$beta <- list(
  kernel = function(y, x, h) {
    rho <- function(y, h) {
      2 * h^2 + 2.5 - sqrt(4 * h^4 + 6 * h^2 + 2.25 - y^2 - y / h)
    }

    N <- length(y)
    cut_points <- c(2 * h, 1 - 2 * h)
    sectioning <- findInterval(y, cut_points)

    y0 <- y[sectioning == 0]
    y1 <- y[sectioning == 1]
    y2 <- y[sectioning == 2]

    par1 <- c(rho(y0, h), y1 / h, y2 / h)
    par2 <- c((1 - y0) / h, (1 - y1) / h, rho(1 - y2, h))

    h * dbeta(x, par1, par2)
  },
  support = c(0, 1)
)

kernel_environment$beta_biased <- list(
  kernel  = function(y, x, h) h * dbeta(x, y / h + 1, (1 - y) / h + 1),
  support = c(0, 1)
)

## ---------------------------------------------------------------------------
## Accessors.
## ---------------------------------------------------------------------------

#' Helper function that gets a kernel function for kdensity.
#'
#' @keywords internal
#' @param kernel_str a string specifying which kernel to use.
#' @return a kernel function of the format k(u) with integral normalized
#' to 1.

get_kernel <- function(kernel_str) {
  assert_(is.character(kernel_str))

  kernel <- kernel_environment[[kernel_str]]

  msg <- paste0("The supplied kernel ('", kernel_str, "') is not implemented.")
  assert_(!is.null(kernel), msg = msg)

  kernel
}

#' Add a new kernel to `kernels_environment`.
#'
#' @keywords internal
#' @param kernel_str A string giving the name of the density.
#' @param kernel The kernel function.
#' @return None.

add_kernel <- function(kernel_str, kernel) {
  assert_(is.character(kernel_str))
  assert_(all(kernel_str == make.names(kernel_str)),
    msg = "The name of the kernel is not valid. Use a short, valid name. (E.g. kdensity(x, kernel = gaussian), where gaussian is a predefined kernel function.)"
  )

  list_msg <- paste0("The kernel ('", kernel_str, "') must be a list.")
  assert_(is.list(kernel), msg = list_msg)

  ## Checks for the right elements in kernel.
  density_msg <- paste0("The kernel ('", kernel_str, "') must contain a function named 'kernel'.")
  support_msg <- paste0("The kernel ('", kernel_str, "') must contain a function named 'support'.")
  assert_(!is.null(kernel$kernel), msg = density_msg)
  assert_(!is.null(kernel$support), msg = support_msg)

  kernel_environment[[kernel_str]] <- kernel
}

# Anchors fully-qualified references for R CMD check's import scanner;
# the modern code path in starts.R uses eval(parser(...)) which is invisible
# to static analysis.

# nocov start
.kdensity_imports_anchor <- function() {
  univariateML::univariateML_models
}
# nocov end

context("kernels")

t_without_kernel <- list(
  support = c(-Inf, Inf)
)

t_without_support <- list(
  kernel = dt
)

t <- list(
  kernel = dt,
  support = c(-Inf, Inf)
)

expect_equal(get_kernel("normal")$sd, 1)
expect_silent(add_kernel("t", t))
expect_error(add_kernel("t", t_without_kernel))
expect_error(add_kernel("t", t_without_support))
expect_error(get_kernel(5))
expect_error(get_kernel("no_kernel"))

assert_ <- function(..., msg = "Assertion failed.") {
  conditions <- list(...)
  ok <- all(vapply(conditions, function(condition) isTRUE(all(condition)), logical(1)))

  if (!ok) {
    stop(msg, call. = FALSE)
  }

  invisible(TRUE)
}

#' Fill in missing kernel, start or support given the supplied values.
#'
#' This function takes the supplied values of kernel, start, and support
#' and fills in the non-supplied ones. It also handles inconsistencies,
#' such as providing a support on (-Inf, Inf) but a kernel on (0, Inf).
#'
#' The `kernel` and `start` parameters are either strings or
#' adhering to the kernel/start list structure. `support` is a
#' numeric vector of length two.
#'
#' @keywords internal
#' @param kernel Supplied kernel; string or list.
#' @param start Supplied parametric start; string or list.
#' @param support Binary vector.
#' @return a list with members kernel, kernel_str, start, start_str,
#' and support.

get_kernel_start_support <- function(kernel, start, support) {
  ## First we handle the parametric start. This is easy, since its default
  ## value equals uniform.

  if (!is.null(start)) {
    start_str <- start
    start <- get_start(start)
  } else {
    start_str <- "uniform"
    start <- get_start("uniform")
  }

  if (is.null(kernel) & is.null(support)) {
    ## No arguments will give you the stats::density behaviour by default.
    return(list(
      kernel = get_kernel("gaussian"),
      kernel_str = "gaussian",
      start = start,
      start_str = start_str,
      support = start$support
    ))
  }


  ## What happens when kernel is also filled in?

  if (is.null(kernel) & !is.null(support)) {
    ## I match a kernel with a support.
    if (all(support == c(-Inf, Inf))) {
      kernel <- "gaussian"
    } else if (all(support == c(0, 1))) {
      kernel <- "gcopula"
    } else if (all(support == c(0, Inf))) {
      kernel <- "gamma"
    } else {
      message("The supplied support does not exactly match any standard kernels.")
      l <- support[1]
      u <- support[2]
      if (l < 0) {
        kernel <- "gaussian"
      } else {
        if (u < 1) {
          kernel <- "gcopula"
        } else {
          kernel <- "gamma"
        }
      }
    }
  }

  ## The next step is to fill in the kernel if it is non-NULL:
  if (!is.null(kernel)) {
    kernel_str <- kernel
    kernel <- get_kernel(kernel)
  }

  if (!is.null(kernel) & is.null(support)) {
    ## The resulting support must be compatible with both the kernel
    ## and the start.
    l <- max(start$support[1], kernel$support[1])
    u <- min(start$support[2], kernel$support[2])
    return(list(
      kernel = kernel,
      kernel_str = kernel_str,
      start = start,
      start_str = start_str,
      support = c(l, u)
    ))
  }

  return(list(
    kernel = kernel,
    kernel_str = kernel_str,
    start = start,
    start_str = start_str,
    support = support
  ))
}


#' Checks compatibility between supports.
#'
#' The supplied support must never be larger than the support of
#' the parametric start / kernel.
#' @keywords internal
#' @param kernel,start,support The kernel, start and support to check.
#' @return None.
support_compatible <- function(kernel, start, support) {
  assert_(kernel$support[1] <= support[1],
    msg =
      "The lower end point of the support is smaller than the lower end point of the 'kernel support'."
  )

  assert_(kernel$support[2] >= support[2],
    msg =
      "The upper end point of the support is larger than the upper end point of the 'kernel support'."
  )

  assert_(start$support[1] <= support[1],
    msg =
      "The lower end point of the support is smaller than the lower end point of the 'start support'."
  )

  assert_(start$support[2] >= support[2],
    msg =
      "The upper end point of the support is larger than the upper end point of the 'start support'."
  )
}

context("utility functions")

expect_silent(assert_(TRUE))
expect_silent(assert_(TRUE, TRUE))
expect_error(assert_(FALSE), "Assertion failed.")
expect_error(assert_(TRUE, FALSE, msg = "custom message"), "custom message")

#' Parametric starts
#'
#' A parametric start is a density function with an associated estimator which
#' is used as a starting point in `kdensity`. Several parametric starts
#' are implemented, all with maximum likelihood estimation. Custom-made
#' parametric starts are possible, see the Structure section.
#'
#' @usage NULL
#' @format NULL
#' @section Structure:
#'   The parametric start contains three elements: The density function, an
#'   estimation function, and the support of the density. The parameters of
#'   the density function must partially match the parameters of the estimator
#'   function. The estimator function takes one argument, a numeric vector,
#'   which is passed from `kdensity`.
#'
#' @section Supported parametric starts: `kdensity` supports more than
#'    20 built-in starts from the [univariateML][univariateML::univariateML-package] package, see
#'    `univariateML::univariateML_models` for a list. Densities with variable
#'    support, `power`, are not supported. The `pareto` density has its
#'    support fixed to `(1,Inf)`. The
#'    options `uniform, constant` makes `kdensity` estimate a kernel
#'    density without parametric starts.
#' @examples start_exponential <- list(
#'   density = stats::dexp,
#'   estimator = function(data) {
#'     c(rate = 1 / mean(data))
#'   },
#'   support = c(0, Inf)
#' )
#'
#' start_inverse_gaussian <- list(
#'   density = extraDistr::dwald,
#'   estimator = function(data) {
#'     c(
#'       mu = mean(data),
#'       lambda = mean(1 / data - 1 / mean(data))
#'     )
#'   },
#'   support = c(0, Inf)
#' )
#'
#' @seealso [kdensity()]; [kernels()]; [bandwidths()]
#' @name parametric_starts
NULL

parser <- function(str) parse(text = str)[[1]]

get_univariate_ml_support <- function(meta) {
  support <- meta$support

  if (isS4(support)) {
    return(list(
      bounds = support@.Data[1, ],
      type = support@type
    ))
  }

  list(
    bounds = support$bounds,
    type = support$type
  )
}

density_namespace_available <- function(density) {
  if (!grepl("::", density, fixed = TRUE)) {
    return(TRUE)
  }

  package_name <- sub("::.*", "", density)
  requireNamespace(package_name, quietly = TRUE)
}

get_density_and_support <- function(fun) {
  if (utils::packageVersion("univariateML") >= "1.5") {
    meta <- "univariateML::univariateML_metadata"
    meta <- eval(parser(paste0(meta,"[[paste0(\"ml\", fun)]]")))
    density <- meta$density
    support <- get_univariate_ml_support(meta)$bounds
    return(list(density = eval(parser(density)), support = support))
  }

  # nocov start
  fun <- eval(parser(paste0("univariateML::ml", fun)))

  for (i in seq(length(body(fun)))) {
    if (length(body(fun)[[i]]) > 1) {
      if (body(fun)[[i]][[2]] == 'attr(object, "density")') {
        density <- body(fun)[[i]][[3]]
      } else if (body(fun)[[i]][[2]] == 'attr(object, "support")') {
        support <- body(fun)[[i]][[3]]
      }
    }
  }
  list(density = eval(parser(density)), support = support)
  # nocov end
}

if (utils::packageVersion("univariateML") >= "1.5") {
  meta <- "univariateML::univariateML_metadata"
  densities <- names(Filter(function(x) {
    get_univariate_ml_support(x)$type == "R" &&
      density_namespace_available(x$density)
  }, eval(parser(meta))))
  densities <- unname(sapply(densities, function(x) substring(x, 3)))
} else {
  densities <- univariateML::univariateML_models
}

starts <- lapply(densities, function(name) {
  c(
    estimator = eval(parser(paste0("univariateML::ml", name))),
    get_density_and_support(name)
  )
})

names(starts) <- densities

## Some densities have variable supports, which is not supported yet.
starts$pareto <- list(
  density   = function(x, alpha) alpha * x^(-alpha - 1),
  estimator = function(x) 1 / mean(log(x)),
  support   = c(1, Inf)
)

starts$power <- NULL

## The uniform distribution is interpreted as uniform over the real line.
starts$unif <- list(
  density   = function(x) rep(1, length(x)),
  estimator = function(data) NULL,
  support   = c(-Inf, Inf)
)

starts$constant <- starts$unif
starts$uniform <- starts$unif

## Aliases for densities.

starts$gaussian <- starts$norm
starts$normal <- starts$norm
starts$exponential <- starts$exp
starts$lognormal <- starts$lnorm
starts$inverse_gaussian <- starts$invgauss
starts$wald <- starts$invgauss

## Make starts_environments with evaled support.
for (i in seq_along(starts)) {
  starts[[i]]$support <- eval(starts[[i]]$support)
}

starts_environment <- as.environment(starts)

## ---------------------------------------------------------------------------
## Accessors.
## ---------------------------------------------------------------------------

#' Get densities and estimators from strings.
#'
#' @keywords internal
#' @param start_str A string specifying the density of interest.
#' @return A list of two functions.

get_start <- function(start_str) {
  assert_(is.character(start_str))

  parametric_start <- starts_environment[[start_str]]

  msg <- paste0("The supplied parametric start ('", start_str, "') is not implemented.")
  assert_(!is.null(parametric_start), msg = msg)

  parametric_start
}

#' Add a new parametric start to `starts_environment`.
#'
#' @keywords internal
#' @param start_str A string giving the name of the density.
#' @param start The parametric start function.
#' @return None.

add_start <- function(start_str, start) {
  assert_(is.character(start_str))
  assert_(all(start_str == make.names(start_str)),
    msg = "The name of the parametric start is not valid. Use a short, valid name. (E.g. kdensity(x, start = gaussian), where gaussian is a predefined start function.)"
  )

  list_msg <- paste0("The parametric start ('", start_str, "') must be a list.")
  assert_(is.list(start), msg = list_msg)

  ## Checks for the right elements in start.
  density_msg <- paste0("The parametric start ('", start_str, "') must contain a function named 'density'.")
  estimator_msg <- paste0("The parametric start ('", start_str, "') must contain a function named 'estimator'.")
  support_msg <- paste0("The parametric start ('", start_str, "') must contain a vector named 'support'.")

  assert_(!is.null(start$density), msg = density_msg)
  assert_(!is.null(start$estimator), msg = estimator_msg)
  assert_(!is.null(start$support), msg = support_msg)

  assign(start_str, start, envir = starts_environment)
}

context("parametric starts")

t_without_estim <- list(
  density = dt,
  support = c(-Inf, Inf)
)

t_without_support <- list(
  density = dt,
  estimator = function(x) NULL
)

t_without_density <- list(
  estimator = function(x) NULL,
  support = c(-Inf, Inf)
)

t <- list(
  density = dt,
  estimator = function(x) NULL,
  support = c(-Inf, Inf)
)

expect_equal(get_start("normal")$support, c(-Inf, Inf))
expect_silent(add_start("t", t))
expect_error(add_start("t", t_without_estim))
expect_error(add_start("t", t_without_support))
expect_error(add_start("t", t_without_density))
expect_error(get_start(5))
expect_error(get_start("no_start"))

univariateML_densities <- c("norm", "lnorm", "weibull", "invgauss", "beta")

expect_true(density_namespace_available("dnorm"))
expect_true(density_namespace_available("stats::dnorm"))
expect_false(density_namespace_available("definitelymissingpkg::dfoo"))

methods::setClass("MockSupportKdensityCoverage", contains = "matrix", slots = c(type = "character"))
mock_support <- methods::new(
  "MockSupportKdensityCoverage",
  matrix(c(-Inf, Inf), nrow = 1),
  type = "R"
)
expect_equal(
  get_univariate_ml_support(list(support = mock_support)),
  list(bounds = c(-Inf, Inf), type = "R")
)

# There are some exceptions we won't test.

exceptions <- c("pareto", "unif", "lomax")

generate_random <- function(n, support) {
  if (support == "c(-Inf, Inf)") {
    return(rnorm(n))
  }
  if (support == "c(0, Inf)") {
    return(rgamma(n, 4, 4))
  }
  if (support == "c(1, Inf)") {
    return(rexp(n) + 1)
  }
  if (support == "c(0, 1)") {
    return(runif(n))
  }
  stop()
}

# Actual testing for all non-exceptions.
set.seed(10)
n <- 50

for (name in univariateML_densities) {
  support <- deparse(get_density_and_support(name)$support)
  rands <- generate_random(n, support)
  kde <- kdensity(rands, start = name)
  coef(kde)
  logLik(kde)
  AIC(kde)
}

# For Lomax
rands <- extraDistr::rlomax(n, 2, 3)
kde <- kdensity(rands, start = "lomax")
coef(kde)
logLik(kde)
AIC(kde)

bw_environment <- new.env(hash = FALSE)

#' Bandwidth Selectors
#'
#' The available options for bandwidth selectors, passed as the `bw`
#' argument to `kdensity`.
#'
#' The bandwidth functions are not exported. They are members of the
#' environment `bw_environments`, and can be accessed by
#' `kdensity:::bw_environments`.
#'
#' @param x The input data.
#' @param kernel_str A string specifying the kernel, e.g. "gaussian."
#' @param start_str A string specifying the parametric start, e.g. "normal".
#' @param support The domain of definition for the kernel. (-Inf, Inf) for
#' symmetric kernels.
#'
#' @section Bandwidth selectors:
#'    `"nrd0", "nrd", "bcv", "SJ"`: Bandwidth selectors from `stats`.
#'    They are documented in `[bandwidth][stats::bandwidth] stats:bandwidth`.
#'    "nrd0" is the standard bandwidth selector for symmetric kernels with
#'    constant parametric starts.
#'
#'    `"ucv"`: Unbiased cross validation. The standard option for
#'    asymmetric kernels.
#'
#'    `"HS"`: Hallberg Szabadváry's closed-form reference rule for the
#'    beta kernel on the unit interval. This is the default for
#'    `kernel = "beta"` with a uniform or constant start.
#'
#'    `"RHE"`: Selector for parametric starts with a symmetric kernel,
#'    based on a reference rule with Hermite polynomials.
#'    Described in Hjort & Glad (1995). The default method in `kdensity` when a parametric
#'    start is supplied and the kernel is symmetric.
#'
#'    `"JH"`: Selector for the Gaussian copula kernel, based on
#'    normal reference rule. Described in Jones & Henderson. The default method when
#'    the `gcopula` kernel is used in `kdensity`.
#'
#'
#' @section Structure:
#'    The bandwidth selector is a function of four arguments: The data
#'    `x`, a kernel string `kernel`, a start string `start`,
#'    and a support vector `support`. To obtain the functions associated
#'    with these strings, use `get_kernel` and `get_start`. The
#'    function should return a double.
#'
#' @seealso [kdensity()], [stats::bandwidth.kernel()] for the
#'    bandwidth selectors of [stats::density()]. In addition,
#'    [kernels()]; [parametric_starts()]
#'
#' @examples
#' ## Not a serious bandwidth function.
#' silly_width <- function(x, kernel = NULL, start = NULL, support = NULL) {
#'   rexp(1)
#' }
#' kdensity(mtcars$mpg, start = "gumbel", bw = silly_width)
#' @references
#' Jones, M. C., and D. A. Henderson. "Kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.
#' Hjort, Nils Lid, and Ingrid K. Glad. "Nonparametric density estimation with a parametric start." The Annals of Statistics (1995): 882-904.
#' Hallberg Szabadváry, Johan. "A Fast, Closed-Form Bandwidth Selector for the Beta Kernel Density Estimator." arXiv preprint arXiv:2601.19553 (2026).
#' @name bandwidths
NULL

bw_environment$JH <- function(x, kernel = NULL, start = NULL, support = NULL) {
  ## The data is transfomed through qnorm, with singularities removed.
  transformed_x <- stats::qnorm(x)
  transformed_x <- transformed_x[transformed_x != Inf & transformed_x != -Inf]
  sigma <- stats::sd(transformed_x)
  mu <- mean(transformed_x)
  n <- length(transformed_x)
  min(sigma * (2 * mu^2 * sigma^2 + 3 * (1 - sigma^2)^2)^(-1 / 5) * n^(-1 / 5), 0.5)
}

bw_environment$RHE <- function(x, kernel = NULL, start = NULL, support = NULL) {
  n <- length(x)
  mu <- mean(x)
  sigma <- stats::sd(x)
  z <- (x - mu) / sigma

  ## Probabilist's Hermite polynomials He_j(sqrt(2) * z) for j = 2..5.
  u <- sqrt(2) * z
  u2 <- u * u
  he2 <- u2 - 1
  he3 <- u * (u2 - 3)
  he4 <- u2 * (u2 - 6) + 3
  he5 <- u * (u2 * (u2 - 10) + 15)

  weight <- sqrt(2) * exp(-z^2 / 2)
  delta <- c(
    mean(weight * he2),
    mean(weight * he3),
    mean(weight * he4),
    mean(weight * he5)
  )

  (1 / 4)^(1 / 5) *
    (delta[1]^2 + delta[2]^2 + delta[3]^2 / 2 + delta[4]^2 / 6)^(-1 / 5) *
    sigma * n^(-1 / 5)
}

bw_environment$nrd0 <- function(data, kernel, start, support) stats::bw.nrd0(data)
bw_environment$nrd <- function(data, kernel, start, support) stats::bw.nrd(data)
bw_environment$bcv <- function(data, kernel, start, support) stats::bw.bcv(data)
bw_environment$SJ <- function(data, kernel, start, support) stats::bw.SJ(data)

bw_environment$HS <- function(x, kernel = NULL, start = NULL, support = NULL) {
  if (!is.numeric(x)) {
    stop("'x' must be a numeric vector.")
  }

  if (length(x) < 2L) {
    stop("'x' must have at least 2 observations.")
  }

  if (any(x < 0 | x > 1, na.rm = TRUE)) {
    stop("All values in 'x' must be in [0, 1].")
  }

  tryCatch(
    compute_hs_bandwidth(x),
    error = function(error) {
      warning(
        paste0(
          "Bandwidth selector 'HS' failed: ",
          conditionMessage(error),
          ". Falling back to 'ucv'."
        ),
        call. = FALSE
      )
      bw_environment$ucv(x, kernel = kernel, start = start, support = support)
    }
  )
}

bw_environment$ucv <- function(x, kernel = NULL, start = NULL, support = NULL) {
  ## We check for the combination start == "uniform" and kernel == "gaussian",
  ## as this is handled by stats::density's related functions.

  if (!is.null(start)) {
    if (start == "constant" | start == "uniform") {
      if (!is.null(get_kernel(kernel)$sd)) {
        return(stats::bw.ucv(x))
      }
    }
  }


  ## If we are here, we must do our own cross-validation.
  kernel_obj <- get_kernel(kernel)
  start_obj <- get_start(start)
  kernel_fun <- kernel_obj$kernel
  start_density <- start_obj$density
  start_estimator <- start_obj$estimator
  start_support <- start_obj$support
  n <- length(x)

  # Name of the variable where the density is evaluated. Typically x.
  x_name <- names(formals(start_density))[1]

  full_parameters <- start_estimator(x)

  arguments <- list()
  arguments[[1]] <- x
  names(arguments)[1] <- x_name
  arguments <- append(arguments, as.list(full_parameters))

  dstart <- function(data, parameters) {
    arguments[[1]] <- data
    if (length(parameters) > 0) {
      for (i in 1:length(parameters)) arguments[[i + 1]] <- parameters[[i]]
    }
    do.call(start_density, arguments)
  }

  param_loo <- lapply(1:n, function(i) start_estimator(x[-i]))

  parametric_start_vector <- function(data) dstart(data, full_parameters)

  parametric_start_data <- parametric_start_vector(x)

  obj_func <- function(h) {
    kdensity_sqrd <- kdensity_sq(x,
      h = h,
      kernel_fun = kernel_fun,
      parametric_start_data = parametric_start_data,
      parametric_start_vector = parametric_start_vector,
      parametric_start = start_density,
      support = support
    )

    term1 <- stats::integrate(kdensity_sqrd,
      lower = support[1],
      upper = support[2]
    )$value

    term2_vec <- rep(0, n)
    for (i in 1:n) {
      # Will not work fro asymmetric? Difference between x and y in kernel.
      term2_vec[i] <- mean(1 / h * kernel_fun(x[-i], x[i], h) /
        dstart(x[-i], param_loo[[i]])) *
        dstart(x[i], param_loo[[i]])
    }
    term2 <- 2 * mean(term2_vec)

    obj_func_value <- term1 - term2
    return(obj_func_value)
  }


  # The range of allowable bandwidths vary from kernel to kernel.

  eps <- 10^-10

  if (kernel == "gcopula" | kernel == "beta" | kernel == "beta_biased") {
    using_str <- "JH"
    using <- bw_environment$JH(x)
    lower <- 1 / 4 * using
    upper <- 1 / 4 - eps
  } else if (start == "constant" | start == "uniform") {
    using_str <- "nrd0"
    using <- stats::bw.nrd0(x)
    lower <- 1 / 5 * using
    upper <- 5 * using
  } else {
    using_str <- "RHE"
    using <- bw_environment$RHE(x)
    lower <- 1 / 5 * using
    upper <- 5 * using
  }

  bw <- tryCatch(stats::optimize(obj_func, lower = lower, upper = upper, tol = 0.0001)$minimum,
    error = function(e) {
      warning(paste0("Integration failed when finding bandwidth using 'ucv'. Using '", using_str, "' instead."), call. = FALSE)
      using
    }
  )
  return(bw)
}

## ---------------------------------------------------------------------------
## Internal helpers used by the selectors above.
## ---------------------------------------------------------------------------

kdensity_sq <- function(x, h, kernel_fun, parametric_start, parametric_start_data,
                        parametric_start_vector, support) {
  normalization <- 1

  pre_function <- function(y) {
    sapply(y, function(y) mean(1 / h * kernel_fun(y, x, h) / parametric_start_data) * parametric_start_vector(y))
  }

  normalization <- tryCatch(stats::integrate(pre_function, lower = support[1], upper = support[2])$value,
    error = function(e) {
      stop("Normalization error: The function will not integrate. Two common causes are: 1.) The kernel is non-smooth, try a smooth kernel if possible. 2.) The supplied support is incorrect.")
    }
  )

  return_function <- function(y) {
    n <- length(y)
    parametric_start_vector_y <- parametric_start_vector(y)
    sapply(1:n, function(i) {
      (1 / h * mean(kernel_fun(y[i], x, h) * parametric_start_vector_y[i] / parametric_start_data) / normalization)^2
    })
  }
  return_function
}

compute_hs_bandwidth <- function(x) {
  x <- x[!is.na(x)]
  n <- length(x)
  x_interior <- x[x > 0 & x < 1]

  if (length(x_interior) == 0L) {
    stop("No data strictly within (0, 1).")
  }

  mu <- mean(x_interior)
  variance <- mean((x_interior - mu)^2)

  if (variance == 0) {
    stop("Sample variance is zero.")
  }

  common <- mu * (1 - mu) / variance - 1
  alpha <- mu * common
  beta <- (1 - mu) * common

  bandwidth <- NA_real_
  use_fallback <- !(alpha > 1.5 && beta > 1.5 && (alpha + beta) > 3)

  if (!use_fallback) {
    log_numerator <- log(2 * alpha + 2 * beta - 5) +
      log(2 * alpha + 2 * beta - 3) +
      lgamma(2 * alpha + 2 * beta - 6) +
      lgamma(alpha) +
      lgamma(beta) +
      lgamma(alpha - 0.5) +
      lgamma(beta - 0.5)

    denominator_term_1 <- (alpha - 1) * (beta - 1)
    denominator_term_2 <- 6 - 4 * beta + alpha * (3 * beta - 4)

    log_denominator <- log(denominator_term_1) +
      log(denominator_term_2) +
      lgamma(2 * alpha - 3) +
      lgamma(2 * beta - 3) +
      lgamma(alpha + beta) +
      lgamma(alpha + beta - 1)

    log_factor <- log(2) + log(n) + 0.5 * log(pi)
    bandwidth <- exp((2 / 5) * (log_numerator - log_denominator - log_factor))
  }

  if (use_fallback) {
    beta_variance <- alpha * beta / ((alpha + beta)^2 * (alpha + beta + 1))
    beta_skewness <- 2 * (beta - alpha) * sqrt(alpha + beta + 1) /
      ((alpha + beta + 2) * sqrt(alpha * beta))
    beta_kurtosis <- 6 * ((alpha - beta)^2 * (alpha + beta + 1) -
      alpha * beta * (alpha + beta + 2)) /
      (alpha * beta * (alpha + beta + 2) * (alpha + beta + 3))

    scale <- sqrt(beta_variance)
    correction <- 1 + abs(beta_skewness) + abs(beta_kurtosis)
    bandwidth <- scale / correction * n^(-0.4)

    warning(
      "MISE rule not applicable; using the HS fallback heuristic.",
      call. = FALSE
    )
  }

  bandwidth
}

## ---------------------------------------------------------------------------
## Accessors.
## ---------------------------------------------------------------------------

#' Get bandwidth functions from string.
#'
#' @keywords internal
#' @param bw_str a string specifying the density of interest.
#' @return a bandwidth function.
get_bw <- function(bw_str) {
  assert_(is.character(bw_str))

  bw <- bw_environment[[bw_str]]

  msg <- paste0("The supplied bandwidth function ('", bw_str, "') is not implemented.")
  assert_(!is.null(bw), msg = msg)

  bw
}


#' Get a bandwidth string when 'bw' is unspecified.
#'
#' @keywords internal
#' @param kernel_str a kernel string
#' @param start_str a parametric start string.
#' @param support the support.
#' @return a bandwidth string.

get_standard_bw <- function(kernel_str, start_str, support) {
  if (kernel_str == "gcopula" & (start_str == "constant" |
    start_str == "uniform")) {
    bw <- "JH"
  } else if (kernel_str == "beta" & (start_str == "constant" |
    start_str == "uniform")) {
    bw <- "HS"
  } else if (start_str != "constant" & start_str != "uniform") {
    if (!is.null(get_kernel(kernel_str)$sd)) {
      bw <- "RHE"
    } else {
      bw <- "ucv"
    }
  } else if (!is.null(get_kernel(kernel_str)$sd)) {
    bw <- "nrd0"
  } else {
    bw <- "ucv"
  }

  bw
}

context("bandwidths")

expect_error(get_bw(5))
expect_error(get_bw("no_kernel"))
expect_silent(get_bw("HS"))

expect_equal(get_standard_bw(kernel_str = "gcopula", start_str = "uniform"), "JH")
expect_equal(get_standard_bw(kernel_str = "gcopula", start_str = "constant"), "JH")
expect_equal(get_standard_bw(kernel_str = "beta", start_str = "uniform"), "HS")
expect_equal(get_standard_bw(kernel_str = "beta", start_str = "constant"), "HS")
expect_equal(get_standard_bw(kernel_str = "beta", start_str = "gaussian"), "ucv")
expect_equal(get_standard_bw(kernel_str = "normal", start_str = "normal"), "RHE")
expect_equal(get_standard_bw(kernel_str = "uniform", start_str = "normal"), "RHE")
expect_equal(get_standard_bw(kernel_str = "gamma", start_str = "normal"), "ucv")
expect_equal(get_standard_bw(kernel_str = "gamma", start_str = "uniform"), "ucv")
expect_equal(get_standard_bw(kernel_str = "epanechnikov", start_str = "constant"), "nrd0")
expect_equal(get_standard_bw(kernel_str = "triangular", start_str = "uniform"), "nrd0")

set.seed(1)
regular_beta_sample <- rbeta(200, 2, 5)
hs_value <- get_bw("HS")(regular_beta_sample, "beta", "uniform", c(0, 1))
expect_gt(hs_value, 0)
## Regression pin for the closed-form HS path; recomputed manually from
## the alpha/beta moment fit at seed = 1.
expect_equal(hs_value, 0.0209275823, tolerance = 1e-8)

set.seed(1)
rhe_sample <- rnorm(500)
## Regression pin for the inlined Hermite polynomial RHE selector;
## matches the previous EQL-based implementation to ~1e-12.
expect_equal(get_bw("RHE")(rhe_sample), 0.4314342622, tolerance = 1e-8)

set.seed(1)
boundary_beta_sample <- rbeta(200, 0.5, 0.5)
expect_warning(
  get_bw("HS")(boundary_beta_sample, "beta", "uniform", c(0, 1)),
  "fallback heuristic"
)

expect_error(get_bw("HS")("not numeric", "beta", "uniform", c(0, 1)), "'x' must be a numeric vector.")
expect_error(get_bw("HS")(0.5, "beta", "uniform", c(0, 1)), "at least 2 observations")
expect_error(get_bw("HS")(c(-0.1, 0.2), "beta", "uniform", c(0, 1)), "must be in \\[0, 1\\]")
expect_error(compute_hs_bandwidth(c(0, 1, NA)), "No data strictly within")
expect_error(compute_hs_bandwidth(c(0.4, 0.4, 0.4)), "Sample variance is zero.")

expect_error(
  kdensity_sq(
    x = c(0.2, 0.8),
    h = 0.1,
    kernel_fun = function(y, x, h) stop("boom"),
    parametric_start = function(x) rep(1, length(x)),
    parametric_start_data = c(1, 1),
    parametric_start_vector = function(y) rep(1, length(y)),
    support = c(0, 1)
  ),
  "Normalization error: The function will not integrate"
)

expect_error(
  kdensity(mtcars$mpg, kernel = "uniform", start = "gamma"),
  "Normalization error: The function will not integrate.Two common causes are: 1.) The kernel is non-smooth, try a smooth kernel if possible. 2.) The supplied support is incorrect."
)

### ===========================================================================
### GENERICS
### This package supports the following generics:
### -- plot, points, lines
### -- summary, print
### ===========================================================================

#' @export
`[[.kdensity` <- function(x, i) {
  allowed_arg <- c(
    "x", "bw_str", "bw", "adjust", "h", "kernel_str", "kernel",
    "start", "start_str", "support", "data.name", "n", "range",
    "has.na", "na.rm", "normalized", "call", "estimates", "logLik"
  )

  i <- match.arg(i, allowed_arg)
  environment(x)[[i]]
}

#' @export
`[[<-.kdensity` <- function(x, i, value) {
  allowed_arg <- c(
    "x", "bw", "adjust", "kernel", "start", "support", "na.rm",
    "normalized"
  )

  i <- match.arg(i, allowed_arg)
  environment(x)$obj_name <- "x"
  args <- list(object = x)
  args[[i]] <- value
  do.call(update.kdensity, args)
  x
}


#' @export
`$<-.kdensity` <- function(x, name, value) {
  x[[name]] <- value
  x
}


#' @export
`$.kdensity` <- function(x, name) {
  x[[name]]
}

#' @export
update.kdensity <- function(object, ...) {
  current <- list(
    x = object$x,
    bw = object$bw_str,
    adjust = object$adjust,
    kernel = object$kernel_str,
    start = object$start_str,
    support = object$support,
    na.rm = object$na.rm,
    normalized = object$normalized
  )

  passed <- list(...)

  ## Part of a hack to make $<- and [[<- work.
  if (!is.null(environment(object)$obj_name)) {
    obj_name <- environment(object)$obj_name
  } else {
    obj_name <- deparse(substitute(object))
  }

  arg_names <- lapply(match.call(expand.dots = TRUE)[-1], deparse)
  args <- utils::modifyList(
    current,
    passed[intersect(names(passed), names(current))],
    keep.null = TRUE
  )
  new_object <- do.call(kdensity, args)

  if ("x" %in% names(arg_names)) {
    data.name <- arg_names$x
  } else {
    data.name <- object$data.name
  }

  call <- call("kdensity",
    x = data.name, adjust = args$adjust,
    kernel = args$kernel, start = args$start, support = args$support
  )

  environment(new_object)$call <- call

  environment(new_object)$data.name <- data.name
  assign(obj_name, new_object, envir = parent.frame())
}

#' @export
coef.kdensity <- function(object, ...) object$estimates

#' @export
logLik.kdensity <- function(object, ...) {
  msg <- "'logLik' only makes sense for kdensity objects with a non-uniform parametric start."
  assert_(object$start_str != "uniform" & object$start_str != "constant", msg = msg)
  val <- object$logLik
  attr(val, "nobs") <- length(object$n)
  attr(val, "df") <- length(stats::coef(object))
  class(val) <- "logLik"
  val
}

#' Supplies a plotting range from a kdensity object.
#'
#' @param obj A kdensity object.
#' @keywords internal
#' @return S vector of size 1000, used for plotting.

get_range <- function(obj) {
  support <- obj$support
  minimum <- obj$range[1]
  maximum <- obj$range[2]
  obj_range <- maximum - minimum
  addition <- obj_range / 10
  xmin <- max(minimum - addition, support[1])
  xmax <- min(maximum + addition, support[2])
  seq(xmin, xmax, length.out = 1000)
}

#' Plot, Lines and Points Methods for Kernel Density Estimation
#'
#' The `plot` method for `kdensity` objects.
#'
#' @export
#' @param x a `kdensity` object.
#' @param range range of x values.
#' @param plot_start logical; if `TRUE`, plots the parametric start instead of the kernel density estimate.
#' @param zero_line logical; if `TRUE`, add a base line at `y = 0`.
#' @param ... further plotting parameters.
#' @return None.
#' @seealso [kdensity()]
#' @examples
#' ## Using the data set "precip" to eye-ball the similarity between
#' ## a kernel fit, a parametric fit, and a kernel with parametric start fit.
#' kde_gamma <- kdensity(precip, kernel = "gaussian", start = "gamma")
#' kde <- kdensity(precip, kernel = "gaussian", start = "uniform")
#'
#' plot(kde_gamma, main = "Annual Precipitation in US Cities")
#' lines(kde_gamma, plot_start = TRUE, lty = 2)
#' lines(kde, lty = 3)
#' rug(precip)
plot.kdensity <- function(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...) {
  plot_helper(x, range, plot_start, zero_line, ptype = "plot", ...)
}

#' @rdname plot.kdensity
#' @export
lines.kdensity <- function(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...) {
  plot_helper(x, range, plot_start, zero_line, ptype = "lines", type = "l", ...)
}

#' @rdname plot.kdensity
#' @export
points.kdensity <- function(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...) {
  plot_helper(x, range, plot_start, zero_line, ptype = "points", type = "p", ...)
}

#' Helper function for the plot methods.
#'
#' A helper function for the plot methods that does most of the work under
#' the hood.
#'
#' @param x A `kdensity` object.
#' @param range An optional range vector; like `x` in `plot.default`.
#' @param plot_start Logical; if `TRUE`, plots the parametric start only.
#' @param zero_line Logical; if `TRUE`, adds a line at `y = 0`.
#' @param ptype The kind of plot to make
#' @param ... Passed to plot.default.
#' @keywords internal
#' @return None.

plot_helper <- function(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ptype = c("plot", "lines", "points"), ...) {
  ptype <- match.arg(ptype)

  if (is.null(range)) range <- get_range(x)

  ## Potential arguments in ellipses are handled here. They are modelled
  ## after the structure of the 'density' function.
  supplied <- list(...)

  bw_string <- NULL
  if (!is.null(x$bw_str)) bw_string <- paste0(" ('", x$bw_str, "')")

  defaults <- list(
    type = "l",
    main = deparse(x$call),
    ylab = "Density",
    xlab = paste0(
      "N = ", x$n, "   Bandwidth = ", formatC(x$bw),
      bw_string
    ),
    lwd = 1
  )

  args <- utils::modifyList(
    defaults,
    supplied,
    keep.null = TRUE
  )
  args$x <- range

  if (plot_start) {
    start <- x$start_str

    msg <- "To use 'plot_start = TRUE', supply a parametric start that is a proper density."
    assert_(!is.null(start), start != "uniform", msg = msg)

    start <- get_start(start)

    parameters <- stats::coef(x)
    parametric_start <- start$density

    args$y <- sapply(range, function(y) {
      do.call(parametric_start, as.list(c("x" = y, parameters)))
    })
  } else {
    args$y <- x(range)
  }

  switch(ptype,
    plot   = do.call(graphics::plot, args),
    lines  = do.call(graphics::lines, args),
    points = do.call(graphics::points, args)
  )

  if (zero_line) graphics::abline(h = 0, lwd = 0.1, col = "gray")

  invisible(x)
}

#' @export
print.kdensity <- function(x, ...) {
  digits <- list(...)$digits
  cat("\nCall:\n", deparse(x$call), "\n\n",
    "Data:      ", x$data.name, " (", x$n, " obs.)\n",
    "Bandwidth: ", formatC(x$bw, digits = digits), " ('", x$bw_str, "')\n",
    "Support:   (", x$support[1], ", ", x$support[2], ")\n",
    "Kernel:    ", x$kernel_str, "\n",
    "Start:     ", x$start_str, "\n\n",
    sep = ""
  )
  invisible(x)
}

#' @export
summary.kdensity <- function(object, ...) {
  digits <- list(...)$digits
  parameters <- object$estimates
  params <- NULL
  if (length(parameters) > 0) {
    params <- c(
      "Parameter estimates:", "\n",
      sapply(1:length(parameters), function(i) paste0(names(parameters)[i], ": ", formatC(parameters[i], digits), "\n")),
      "\n"
    )
  }
  cat("\nCall: \n", deparse(object$call), "\n\n",
    "Data:        ", object$data.name, " (", object$n, " obs.)\n",
    "Bandwidth:   ", formatC(object$bw, digits = digits), " ('", object$bw_str, "')\n",
    "Support:     (", object$support[1], ", ", object$support[2], ")\n",
    "Kernel:      ", object$kernel_str, "\n",
    "Start:       ", object$start_str, "\n",
    "Range:       (", formatC(object$range[1], digits), ", ", formatC(object$range[2], digits), ")\n",
    "NAs in data: ", object$has.na, "\n",
    "Adjustment:  ", object$adjust, "\n\n",
    params,
    sep = ""
  )
  invisible(object)
}

---
title: "Tutorial for `kdensity`"
author: "Jonas Moss"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tutorial for `kdensity`}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Basic usage

`kdensity` is called using a syntax similar to `stats::density`, but with some additional arguments. A call to `kdensity` returns a density function (with class `kdensity`), which can be used as ordinary `R`-functions.

```{r mpg_example_start, echo = TRUE}
library("kdensity")
kde <- kdensity(mtcars$mpg, start = "gumbel", kernel = "gaussian")
kde(20)
```
Hence it can be used to calculate functionals, such as the expectation.

```{r mpg_example_expectation, echo = TRUE}
integrate(function(x) x * kde(x), lower = -Inf, upper = Inf)$value
```

Plotting works as with `stats::density`. If `kdensity` was called with a parametric start, use `plot_start = TRUE` inside a plotting function in order to plot the estimated parametric start density instead of the kernel density.

```{r mpg_example_plot, echo = TRUE,fig.height = 5, fig.width = 5, fig.align = "center"}
plot(kde, main = "Miles per Gallon")
lines(kde, plot_start = TRUE, col = "red")
rug(mtcars$mpg)
```

If the R-package `magrittr` is installed, you can use pipes to plot.

```{r mpg_example_plot_magrittr, echo = TRUE, eval = FALSE}
library("magrittr")
kde %>%
  plot(main = "Miles per Gallon") %>%
  lines(plot_start = TRUE, col = "red")
```

The generics `coef`, `logLik`, `AIC` and `BIC` are supported, but work only on the parametric start. 

```{r mpg_example_generics, echo = TRUE}
coef(kde)
logLik(kde)
AIC(kde)
```

To view information about the kernel density, use the `summary` generic.

```{r mpg_example_summary, echo = TRUE}
summary(kde)
```

Access members of the kernel density with `$` or `[[`.

```{r mpg_example_access, echo = TRUE}
kde$start_str
kde[["x"]]
```

To make changes to the kernel density estimator, you can use the generic `update` function, replace values with `$`, or make a new one.

```{r mpg_example_changes, echo = TRUE}
kde$bw <- "RHE"
update(kde, start = "normal")
```

`kdensity` supports all parametric starts implemented by the package 
`univariateML`. See [its Github page](https://github.com/JonasMoss/univariateML)
for a complete list, or take a look at `univariateML::univariateML_models`.

# Custom densities and kernels
## Parametric starts

You must supply two functions and one vector to `kdensity` in order to use a custom parametric start. The first is a `density` function, the second is a function that estimates the named parameters of the density, and the third is the support of the density. 

```{r}
normal <- list(
  density = dnorm,
  estimator = function(data) {
    c(
      mean = mean(data),
      sd = sd(data)
    )
  },
  support = c(-Inf, Inf)
)
```

The density function must take the data as its first argument, and all its parameters must be named. In addition, the function `estimator` must return a vector containing named parameters that partially match the parameter names of the density function. For instance, the arguments of `dnorm` are `x, mean, sd, log`, where `log = TRUE` means that the logarithm of the density is returned. Since `estimator` returns a named vector with names `mean` and `sd`, the names are completely matched. 

The estimator function doesn't need to be simple, as the next example shows.

### Example: A skew hyperbolic t-distribution start

The built-in data set `LakeHuron` contains annual measurements of the water level of Lake Huron from 1875 to 1972, measured in feet. We will take a look at the distribution of differences in water level across two consecutive years. Since the data are supported on the real line and there is no good reason to assume anything else, we will use a normal start.

```{r, fig2, fig.height = 5, fig.width = 5, fig.align = "center"}
LH <- diff(LakeHuron)
kde_normal <- kdensity(LH, start = "normal")
plot(kde_normal,
  lwd = 2, col = "black",
  main = "Lake Huron differences"
)
```

The density is clearly non-normal. Still, it looks fairly regular, and it should be possible to model it parametrically with some success. In practice, `kdensity` can also work with user-supplied densities and estimators from other packages, as long as you provide a list with `density`, `estimator`, and `support`.

## Kernels

If you want to make custom kernel, you will need to supply the kernel function, with arguments `y, x, h`. Here `x` is the random data you put into `kdensity`, `h` is the final bandwidth, and `y` is the point you want to evaluate at. The kernel is called as `1/h*kernel(y, x, h)`, and should be able to take vector inputs `x` and `y`. In addition to the kernel function, you must supply a `support` argument, which states the domain of definition of the kernel. For instance, the `gcopula` kernel is defined on `c(0, 1)`. In addition, you can optionally supply the standard deviation of the kernel. This is only used for symmetric kernels, and is useful since it makes them comparable. For example, the implementation of the `gaussian` kernel is

```{r}
gaussian <- list(
  kernel  = function(y, x, h) dnorm((y - x) / h),
  sd      = 1,
  support = c(-Inf, Inf)
)
```

Custom kernels can be complicated, and do not have to be symmetric.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/starts.R
\name{get_start}
\alias{get_start}
\title{Get densities and estimators from strings.}
\usage{
get_start(start_str)
}
\arguments{
\item{start_str}{A string specifying the density of interest.}
}
\value{
A list of two functions.
}
\description{
Get densities and estimators from strings.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kernels.R
\name{get_kernel}
\alias{get_kernel}
\title{Helper function that gets a kernel function for kdensity.}
\usage{
get_kernel(kernel_str)
}
\arguments{
\item{kernel_str}{a string specifying which kernel to use.}
}
\value{
a kernel function of the format k(u) with integral normalized
to 1.
}
\description{
Helper function that gets a kernel function for kdensity.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{get_range}
\alias{get_range}
\title{Supplies a plotting range from a kdensity object.}
\usage{
get_range(obj)
}
\arguments{
\item{obj}{A kdensity object.}
}
\value{
S vector of size 1000, used for plotting.
}
\description{
Supplies a plotting range from a kdensity object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{plot.kdensity}
\alias{plot.kdensity}
\alias{lines.kdensity}
\alias{points.kdensity}
\title{Plot, Lines and Points Methods for Kernel Density Estimation}
\usage{
\method{plot}{kdensity}(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...)

\method{lines}{kdensity}(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...)

\method{points}{kdensity}(x, range = NULL, plot_start = FALSE, zero_line = TRUE, ...)
}
\arguments{
\item{x}{a \code{kdensity} object.}

\item{range}{range of x values.}

\item{plot_start}{logical; if \code{TRUE}, plots the parametric start instead of the kernel density estimate.}

\item{zero_line}{logical; if \code{TRUE}, add a base line at \code{y = 0}.}

\item{...}{further plotting parameters.}
}
\value{
None.
}
\description{
The \code{plot} method for \code{kdensity} objects.
}
\examples{
## Using the data set "precip" to eye-ball the similarity between
## a kernel fit, a parametric fit, and a kernel with parametric start fit.
kde_gamma <- kdensity(precip, kernel = "gaussian", start = "gamma")
kde <- kdensity(precip, kernel = "gaussian", start = "uniform")

plot(kde_gamma, main = "Annual Precipitation in US Cities")
lines(kde_gamma, plot_start = TRUE, lty = 2)
lines(kde, lty = 3)
rug(precip)
}
\seealso{
\code{\link[=kdensity]{kdensity()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{support_compatible}
\alias{support_compatible}
\title{Checks compatibility between supports.}
\usage{
support_compatible(kernel, start, support)
}
\arguments{
\item{kernel, start, support}{The kernel, start and support to check.}
}
\value{
None.
}
\description{
The supplied support must never be larger than the support of
the parametric start / kernel.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/starts.R
\name{parametric_starts}
\alias{parametric_starts}
\title{Parametric starts}
\description{
A parametric start is a density function with an associated estimator which
is used as a starting point in \code{kdensity}. Several parametric starts
are implemented, all with maximum likelihood estimation. Custom-made
parametric starts are possible, see the Structure section.
}
\section{Structure}{

The parametric start contains three elements: The density function, an
estimation function, and the support of the density. The parameters of
the density function must partially match the parameters of the estimator
function. The estimator function takes one argument, a numeric vector,
which is passed from \code{kdensity}.
}

\section{Supported parametric starts}{
 \code{kdensity} supports more than
20 built-in starts from the \link[univariateML:univariateML-package]{univariateML} package, see
\code{univariateML::univariateML_models} for a list. Densities with variable
support, \code{power}, are not supported. The \code{pareto} density has its
support fixed to \verb{(1,Inf)}. The
options \verb{uniform, constant} makes \code{kdensity} estimate a kernel
density without parametric starts.
}

\examples{
start_exponential <- list(
  density = stats::dexp,
  estimator = function(data) {
    c(rate = 1 / mean(data))
  },
  support = c(0, Inf)
)

start_inverse_gaussian <- list(
  density = extraDistr::dwald,
  estimator = function(data) {
    c(
      mu = mean(data),
      lambda = mean(1 / data - 1 / mean(data))
    )
  },
  support = c(0, Inf)
)

}
\seealso{
\code{\link[=kdensity]{kdensity()}}; \code{\link[=kernels]{kernels()}}; \code{\link[=bandwidths]{bandwidths()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/starts.R
\name{add_start}
\alias{add_start}
\title{Add a new parametric start to \code{starts_environment}.}
\usage{
add_start(start_str, start)
}
\arguments{
\item{start_str}{A string giving the name of the density.}

\item{start}{The parametric start function.}
}
\value{
None.
}
\description{
Add a new parametric start to \code{starts_environment}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kernels.R
\name{kernels}
\alias{kernels}
\title{Kernel functions}
\description{
Kernel functions are an important part of \code{kdensity}. This document
lists the available built-in functions and the structure of them. Any kernel
in the list can be used in \code{kdensity} by using \code{kernel = "kernel"}
for the intended kernel.
}
\details{
Be careful combining kernels with compact support with parametric starts,
as the normalizing integral typically fails to converge. Use \code{gaussian}
instead.
}
\section{Symmetric kernels}{



\verb{gaussian, normal}: The Gaussian kernel. The default argument when
\code{starts} is supported on R.
\verb{epanechnikov, rectangular (uniform), triangular, biweight, cosine, optcosine}: Standard symmetric kernels, also used in
\code{\link[stats:density]{stats::density()}}.
\verb{tricube, triweight}: Standard symmetric kernels. Not supported by
\code{\link[stats:density]{stats::density()}}.
\code{laplace}: Uses the Laplace density, also known as the double
exponential density.
}

\section{Asymmetric kernels}{



\verb{gamma, gamma_biased}: The gamma kernel of Chen (2000). For use on the positive
half-line. \code{gamma} is the recommended biased-corrected kernel.
\code{gcopula}: The Gaussian copula kernel of Jones & Henderson (2007). For use
on the unit interval.
\verb{beta, beta_biased}: The beta kernel of Chen (1999). For use on the unit interval.
\code{beta} is the recommended bias-corrected kernel.
}

\section{Structure}{

A kernel is a list containing two mandatory elements and one optional
element. The mandatory element '\code{kernel}' is the kernel function.
It takes arguments \verb{y, x, h}, where \code{x} is the data supplied
to \code{kdensity} and \code{y} is the point of evaluation. \code{h} is
the bandwidth. Internally, the kernel function is evaluated as
\code{1/h*kernel(y, x, h)}. It should be vectorized in \code{x}, but
vectorization in \code{y} is not needed.

The second mandatory element is \code{support}, stating the domain of
definition for the kernel. This is used to distinguish kernels on the
unit interval / positive half-line from kernels on R.

\code{sd} is used for symmetric kernels, and states the standard error
of the kernel. This is used to make kernels comparable to the Gaussian
kernel when calculating bandwidths.
}

\examples{
gaussian <- list(
  kernel = function(y, x, h) stats::dnorm((y - x) / h),
  sd = 1,
  support = c(-Inf, Inf)
)

gcopula <- list(
  kernel = function(y, x, h) {
    rho <- 1 - h^2
    inside <- rho^2 * (qnorm(y)^2 + qnorm(x)^2) - 2 * rho * qnorm(y) * qnorm(x)
    exp(-inside / (2 * (1 - rho^2)))
  },
  support = c(0, 1)
)

}
\references{
Chen, Song Xi. "Probability density function estimation using gamma kernels." Annals of the Institute of Statistical Mathematics 52.3 (2000): 471-480.
Jones, M. C., and D. A. Henderson. "Kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.
Chen, Song Xi. "Beta kernel estimators for density functions." Computational Statistics & Data Analysis 31.2 (1999): 131-145.
}
\seealso{
\code{\link[=kdensity]{kdensity()}}; \code{\link[=parametric_starts]{parametric_starts()}};
\code{\link[=bandwidths]{bandwidths()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kernels.R
\name{add_kernel}
\alias{add_kernel}
\title{Add a new kernel to \code{kernels_environment}.}
\usage{
add_kernel(kernel_str, kernel)
}
\arguments{
\item{kernel_str}{A string giving the name of the density.}

\item{kernel}{The kernel function.}
}
\value{
None.
}
\description{
Add a new kernel to \code{kernels_environment}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bandwidths.R
\name{get_standard_bw}
\alias{get_standard_bw}
\title{Get a bandwidth string when 'bw' is unspecified.}
\usage{
get_standard_bw(kernel_str, start_str, support)
}
\arguments{
\item{kernel_str}{a kernel string}

\item{start_str}{a parametric start string.}

\item{support}{the support.}
}
\value{
a bandwidth string.
}
\description{
Get a bandwidth string when 'bw' is unspecified.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kdensity.R
\name{kdensity}
\alias{kdensity}
\title{Parametrically guided kernel density estimation}
\usage{
kdensity(
  x,
  bw = NULL,
  adjust = 1,
  kernel = NULL,
  start = NULL,
  support = NULL,
  na.rm = FALSE,
  normalized = TRUE,
  tolerance = 0.01
)
}
\arguments{
\item{x}{Numeric vector containing the data.}

\item{bw}{A bandwidth function. Can be either a string, a custom-made
function, or a double. The supported bandwidth functions are documented
in \code{\link[=bandwidths]{bandwidths()}}.}

\item{adjust}{An adjustment constant, so that \code{h = adjust*bw*sd}, where \code{sd}
varies with the chosen kernel.}

\item{kernel}{The kernel function. Can be chosen from the list of built-in
kernels or be custom-made. See \code{\link[=kernels]{kernels()}} for details.}

\item{start}{Parametric start. Can be chosen from the list of built-in
parametric starts or be custom-made. See \code{\link[=parametric_starts]{parametric_starts()}} for
details.}

\item{support}{The support of the data. Must be compatible with the supplied
\code{x} and the supplied \code{start} and \code{kernel}. Is used to find the
normalization constant, see \code{normalized}.}

\item{na.rm}{Logical; if \code{TRUE}, \code{NA}s will be removed from \code{x}.}

\item{normalized}{Logical; if \code{TRUE}, the density is normalized.}

\item{tolerance}{Numeric; the relative error to tolerate in normalization.}
}
\value{
\code{kdensity} returns an S3 function object of
\code{\link[base:class]{base::class()}} "kdensity". This is a callable function with the
following elements, accessible by '$':
\describe{
\item{\code{x}}{The data supplied in \code{x}.}
\item{\verb{bw_str, bw, adjust, h}}{The bandwidth function, the resulting
bandwidth, the \code{adjust} argument, and the adjusted
bandwidth.}
\item{\verb{kernel_str, kernel, start, start_str, support}}{Name of the kernel,
the kernel object, name of the parametric start, the start object,
and the support of the density.}
\item{\verb{data.name, n, range, has.na, na.rm, normalized}}{Name of the data, number of
observations, the range of the data, whether the data
\code{x} contained \code{NA} values, whether na.rm is \code{TRUE}
or not, and whether the density is normalized.}
\item{\code{call}}{The \code{call} to \code{kdensity}.}
\item{\code{estimates}}{Named numeric vector containing the parameter
estimates from the parametric start.}
\item{\code{logLik}}{The log-likelihood of the parametric starts. Is \code{NA}
for the uniform start.}

}
}
\description{
\code{kdensity} computes a parametrically guided kernel density estimate
for univariate data. It supports asymmetric kernels and parametric starts
through the \code{kernel} and \code{start} arguments.
}
\details{
The default values for \code{bw}, \code{kernel}, \code{start}, and
\code{support} are interdependent, and are chosen to make sense. E.g.,
the default value for \code{support} when \code{start = beta} is
\code{c(0, 1)}.

The \code{start} argument defaults to \code{uniform}, which corresponds
to ordinary kernel density estimation. The typical default value for
\code{kernel} is \code{gaussian}.

If \code{normalized} is \code{FALSE} and \code{start != "uniform"}, the resulting
density will not integrate to 1 in general.
}
\examples{
## Use gamma kernels to model positive data, the concentration of
## theophylline

concentration <- Theoph$conc + 0.001
plot(kdensity(concentration, start = "gamma", kernel = "gamma", adjust = 1 / 3),
  ylim = c(0, 0.15), lwd = 2, main = "Concentration of theophylline"
)
lines(kdensity(concentration, start = "gamma", kernel = "gaussian"),
  lty = 2, col = "grey", lwd = 2
)
lines(kdensity(concentration, start = "gaussian", kernel = "gaussian"),
  lty = 3, col = "blue", lwd = 2
)
lines(kdensity(concentration, start = "gaussian", kernel = "gamma", adjust = 1 / 3),
  lty = 4, col = "red", lwd = 2
)
rug(concentration)

}
\references{
Hjort, Nils Lid, and Ingrid K. Glad. "Nonparametric density estimation with a parametric start." The Annals of Statistics (1995): 882-904.

Jones, M. C., and D. A. Henderson. "Miscellanea kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.

Chen, Song Xi. "Probability density function estimation using gamma kernels." Annals of the Institute of Statistical Mathematics 52.3 (2000): 471-480.

Silverman, Bernard W. Density estimation for statistics and data analysis. Vol. 26. CRC press, 1986.
}
\seealso{
The \code{stats} package function \code{\link[stats:density]{stats::density()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bandwidths.R
\name{bandwidths}
\alias{bandwidths}
\title{Bandwidth Selectors}
\arguments{
\item{x}{The input data.}

\item{kernel_str}{A string specifying the kernel, e.g. "gaussian."}

\item{start_str}{A string specifying the parametric start, e.g. "normal".}

\item{support}{The domain of definition for the kernel. (-Inf, Inf) for
symmetric kernels.}
}
\description{
The available options for bandwidth selectors, passed as the \code{bw}
argument to \code{kdensity}.
}
\details{
The bandwidth functions are not exported. They are members of the
environment \code{bw_environments}, and can be accessed by
\code{kdensity:::bw_environments}.
}
\section{Bandwidth selectors}{

\verb{"nrd0", "nrd", "bcv", "SJ"}: Bandwidth selectors from \code{stats}.
They are documented in \verb{[bandwidth][stats::bandwidth] stats:bandwidth}.
"nrd0" is the standard bandwidth selector for symmetric kernels with
constant parametric starts.

\code{"ucv"}: Unbiased cross validation. The standard option for
asymmetric kernels.

\code{"HS"}: Hallberg Szabadváry's closed-form reference rule for the
beta kernel on the unit interval. This is the default for
\code{kernel = "beta"} with a uniform or constant start.

\code{"RHE"}: Selector for parametric starts with a symmetric kernel,
based on a reference rule with Hermite polynomials.
Described in Hjort & Glad (1995). The default method in \code{kdensity} when a parametric
start is supplied and the kernel is symmetric.

\code{"JH"}: Selector for the Gaussian copula kernel, based on
normal reference rule. Described in Jones & Henderson. The default method when
the \code{gcopula} kernel is used in \code{kdensity}.
}

\section{Structure}{

The bandwidth selector is a function of four arguments: The data
\code{x}, a kernel string \code{kernel}, a start string \code{start},
and a support vector \code{support}. To obtain the functions associated
with these strings, use \code{get_kernel} and \code{get_start}. The
function should return a double.
}

\examples{
## Not a serious bandwidth function.
silly_width <- function(x, kernel = NULL, start = NULL, support = NULL) {
  rexp(1)
}
kdensity(mtcars$mpg, start = "gumbel", bw = silly_width)
}
\references{
Jones, M. C., and D. A. Henderson. "Kernel-type density estimation on the unit interval." Biometrika 94.4 (2007): 977-984.
Hjort, Nils Lid, and Ingrid K. Glad. "Nonparametric density estimation with a parametric start." The Annals of Statistics (1995): 882-904.
Hallberg Szabadváry, Johan. "A Fast, Closed-Form Bandwidth Selector for the Beta Kernel Density Estimator." arXiv preprint arXiv:2601.19553 (2026).
}
\seealso{
\code{\link[=kdensity]{kdensity()}}, \code{\link[stats:kernel]{stats::bandwidth.kernel()}} for the
bandwidth selectors of \code{\link[stats:density]{stats::density()}}. In addition,
\code{\link[=kernels]{kernels()}}; \code{\link[=parametric_starts]{parametric_starts()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bandwidths.R
\name{get_bw}
\alias{get_bw}
\title{Get bandwidth functions from string.}
\usage{
get_bw(bw_str)
}
\arguments{
\item{bw_str}{a string specifying the density of interest.}
}
\value{
a bandwidth function.
}
\description{
Get bandwidth functions from string.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get_kernel_start_support}
\alias{get_kernel_start_support}
\title{Fill in missing kernel, start or support given the supplied values.}
\usage{
get_kernel_start_support(kernel, start, support)
}
\arguments{
\item{kernel}{Supplied kernel; string or list.}

\item{start}{Supplied parametric start; string or list.}

\item{support}{Binary vector.}
}
\value{
a list with members kernel, kernel_str, start, start_str,
and support.
}
\description{
This function takes the supplied values of kernel, start, and support
and fills in the non-supplied ones. It also handles inconsistencies,
such as providing a support on (-Inf, Inf) but a kernel on (0, Inf).
}
\details{
The \code{kernel} and \code{start} parameters are either strings or
adhering to the kernel/start list structure. \code{support} is a
numeric vector of length two.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{plot_helper}
\alias{plot_helper}
\title{Helper function for the plot methods.}
\usage{
plot_helper(
  x,
  range = NULL,
  plot_start = FALSE,
  zero_line = TRUE,
  ptype = c("plot", "lines", "points"),
  ...
)
}
\arguments{
\item{x}{A \code{kdensity} object.}

\item{range}{An optional range vector; like \code{x} in \code{plot.default}.}

\item{plot_start}{Logical; if \code{TRUE}, plots the parametric start only.}

\item{zero_line}{Logical; if \code{TRUE}, adds a line at \code{y = 0}.}

\item{ptype}{The kind of plot to make}

\item{...}{Passed to plot.default.}
}
\value{
None.
}
\description{
A helper function for the plot methods that does most of the work under
the hood.
}
\keyword{internal}
